@@ -14,7 +14,8 @@ static int	tflag;
 static int	numpmid;
 static pmID	pmidlist[20];
 static const char *namelist[20];
-static double	delta = 500;
+static struct timespec	delta = { 0, 500000 };
+static struct timespec	minus_delta = { 0, -500000 };
 
 static void
 cmpres(int n, pmResult *e, pmResult *g)
@@ -24,10 +25,10 @@ cmpres(int n, pmResult *e, pmResult *g)
     int		err = 0;
 
     if (e->timestamp.tv_sec != g->timestamp.tv_sec ||
-	e->timestamp.tv_usec != g->timestamp.tv_usec) {
-	printf("[sample %d] pmResult.timestamp: expected %ld.%06ld, got %ld.%06ld\n",
-	    n, (long)e->timestamp.tv_sec, (long)e->timestamp.tv_usec,
-	    (long)g->timestamp.tv_sec, (long)g->timestamp.tv_usec);
+	e->timestamp.tv_nsec != g->timestamp.tv_nsec) {
+	printf("[sample %d] pmResult.timestamp: expected %ld.%09ld, got %ld.%09ld\n",
+	    n, (long)e->timestamp.tv_sec, (long)e->timestamp.tv_nsec,
+	    (long)g->timestamp.tv_sec, (long)g->timestamp.tv_nsec);
 	goto FAILED;
     }
     if (e->numpmid != g->numpmid) {
@@ -99,7 +100,7 @@ main(int argc, char **argv)
     pmResult	*resp;
     pmResult	**resvec = (pmResult **)0;
     int		resnum = 0;
-    struct timeval	when;
+    struct timespec	when;
     int		done;
 
     pmSetProgname(argv[0]);
@@ -132,8 +133,14 @@ main(int argc, char **argv)
 	    tflag++;
 	    break;
 
-	case 't':	/* sample interval */
-	    delta = 1000 * atof(optarg);
+	case 't':	/* sample interval (sec) */
+	    pmtimespecFromReal(atof(optarg), &delta);
+	    if (delta.tv_sec > 0) {
+		minus_delta.tv_sec = -delta.tv_sec;
+		minus_delta.tv_nsec = delta.tv_nsec;
+	    }
+	    else
+		minus_delta.tv_nsec = -delta.tv_nsec;
 	    break;
 
 	case '?':
@@ -162,8 +169,8 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    when = loglabel.ll_start;
-    if ((sts = pmSetMode(PM_MODE_INTERP, &when, delta)) < 0) {
+    when = loglabel.start;
+    if ((sts = pmSetMode(PM_MODE_INTERP, &when, &delta)) < 0) {
 	printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
@@ -172,7 +179,7 @@ main(int argc, char **argv)
         printf("%s: Cannot dup context to archive \"%s\": %s\n", pmGetProgname(), archive, pmErrStr(ctx[0]));
         exit(1);
     }
-    if ((sts = pmSetMode(PM_MODE_INTERP, &when, delta)) < 0) {
+    if ((sts = pmSetMode(PM_MODE_INTERP, &when, &delta)) < 0) {
         printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
         exit(1);
     }
@@ -225,7 +232,7 @@ main(int argc, char **argv)
     printf("\nPass 2: backwards scan\n");
     __pmLogReads = 0;
     when = resvec[resnum - 1]->timestamp;
-    if ((sts = pmSetMode(PM_MODE_INTERP, &when, -delta)) < 0) {
+    if ((sts = pmSetMode(PM_MODE_INTERP, &when, &minus_delta)) < 0) {
 	printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
@@ -250,14 +257,14 @@ main(int argc, char **argv)
     printf("\nPass 3: concurrent forwards and backwards scans\n");
     __pmLogReads = 0;
     pmUseContext(ctx[0]);
-    when = loglabel.ll_start;
-    if ((sts = pmSetMode(PM_MODE_INTERP, &when, delta)) < 0) {
+    when = loglabel.start;
+    if ((sts = pmSetMode(PM_MODE_INTERP, &when, &delta)) < 0) {
 	printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
     pmUseContext(ctx[1]);
     when = resvec[resnum - 1]->timestamp;
-    if ((sts = pmSetMode(PM_MODE_INTERP, &when, -delta)) < 0) {
+    if ((sts = pmSetMode(PM_MODE_INTERP, &when, &minus_delta)) < 0) {
 	printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
@@ -305,7 +312,7 @@ main(int argc, char **argv)
 	    i = resnum - 1;
 	when = resvec[i]->timestamp;
 
-	if ((sts = pmSetMode(PM_MODE_INTERP, &when, delta)) < 0) {	
+	if ((sts = pmSetMode(PM_MODE_INTERP, &when, &delta)) < 0) {	
 	    printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
@@ -338,7 +345,7 @@ main(int argc, char **argv)
 	if (i < 0)
 	    i = 0;
 	when = resvec[i]->timestamp;
-	if ((sts = pmSetMode(PM_MODE_INTERP, &when, -delta)) < 0) {
+	if ((sts = pmSetMode(PM_MODE_INTERP, &when, &minus_delta)) < 0) {
 	    printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
@@ -371,7 +378,7 @@ main(int argc, char **argv)
 	    i = resnum - 1;
 
 	when = resvec[0]->timestamp;
-	if ((sts = pmSetMode(PM_MODE_INTERP, &when, delta)) < 0) {	
+	if ((sts = pmSetMode(PM_MODE_INTERP, &when, &delta)) < 0) {	
 	    printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
@@ -386,7 +393,7 @@ main(int argc, char **argv)
 	}
 
 	when = resvec[i]->timestamp;
-	if ((sts = pmSetMode(PM_MODE_INTERP, &when, -delta)) < 0) {	
+	if ((sts = pmSetMode(PM_MODE_INTERP, &when, &minus_delta)) < 0) {	
 	    printf("%s: pmSetMode: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
