@@ -24,13 +24,29 @@ func(void *arg)
     return(NULL);	/* pthread done */
 }
 
+static void
+wait_for_thread(char *name, pthread_t tid)
+{
+    int		sts;
+    char	*msg;
+
+    sts = pthread_join(tid, (void *)&msg);
+    if (sts == 0) {
+	if (msg == PTHREAD_CANCELED)
+	    printf("thread %s: pthread_join: cancelled?\n", name);
+	else if (msg != NULL)
+	    printf("thread %s: pthread_join: %s\n", name, msg);
+    }
+    else
+	printf("thread %s: pthread_join: error: %s\n", name, strerror(sts));
+}
+
 int
 main()
 {
     pthread_t	tid1;
     pthread_t	tid2;
     int		sts;
-    char	*msg;
 
     if (__pmMultiThreaded(PM_SCOPE_DSO_PMDA))
 	printf("main0: is multithreaded\n");
@@ -59,10 +75,8 @@ main()
     else
 	printf("main2: is NOT multithreaded\n");
 
-    pthread_join(tid1, (void *)&msg);
-    if (msg != NULL) printf("tid1: %s\n", msg);
-    pthread_join(tid2, (void *)&msg); 
-    if (msg != NULL) printf("tid2: %s\n", msg);
+    wait_for_thread("tid1", tid1);
+    wait_for_thread("tid2", tid2);
 
     exit(0);
 }
