@@ -127,7 +127,7 @@ add_record(struct timeval *tp, int flags)
 static void
 dump(char *xpect)
 {
-    pmResult	**res;
+    pmResult_v2	**res;
     int		nmissed;
     int		nrecords;
     int		r;
@@ -155,8 +155,8 @@ dump(char *xpect)
 	     * __pmRegisterAnon(), so the anon metrics
 	     * should now be in the PMNS
 	     */
-	    char	*name_flags = "event.flags";
-	    char	*name_missed = "event.missed";
+	    const char	*name_flags = "event.flags";
+	    const char	*name_missed = "event.missed";
 	    int	sts;
 	    sts = pmLookupName(1, &name_flags, &pmid_flags);
 	    if (sts < 0) {
@@ -189,7 +189,7 @@ dump(char *xpect)
 
 	for (r = 0; r < nrecords; r++) {
 	    fprintf(stderr, "pmResult[%d]\n", r);
-	    __pmDumpResult(stderr, res[r]);
+	    __pmDumpResult_v2(stderr, res[r]);
 	}
 	pmFreeEventResult(res);
     }
