@@ -93,7 +93,7 @@ _pcp_complete()
 	arg_regex="-[DhnpZ]"
     ;;
     pmlogcheck)
-	all_args="DlmnSTzvwZ"
+	all_args="DflmnRrSTzvwZ"
 	arg_regex="-[DnSTZ]"
     ;;
     pmlogctl)
