@@ -83,7 +83,7 @@ static const struct {
     { PM_ERR_TIMEOUT,		"PM_ERR_TIMEOUT",
 	"Timeout waiting for a response from PMCD" },
     { PM_ERR_NODATA,		"PM_ERR_NODATA",
-	"Empty archive log file" },
+	"Empty archive file" },
     { PM_ERR_RESET,		"PM_ERR_RESET",
 	"PMCD reset or configuration change" },
     { PM_ERR_NAME,		"PM_ERR_NAME",
@@ -115,15 +115,15 @@ static const struct {
     { PM_ERR_NOTHOST,		"PM_ERR_NOTHOST",
 	"Operation requires context with host source of metrics" },
     { PM_ERR_EOL,		"PM_ERR_EOL",
-	"End of PCP archive log" },
+	"End of PCP archive" },
     { PM_ERR_MODE,		"PM_ERR_MODE",
 	"Illegal mode specification" },
     { PM_ERR_LABEL,		"PM_ERR_LABEL",
-	"Illegal label record at start of a PCP archive log file" },
+	"Illegal label record at start of a PCP archive file" },
     { PM_ERR_LOGREC,		"PM_ERR_LOGREC",
-	"Corrupted record in a PCP archive log" },
+	"Corrupted record in a PCP archive" },
     { PM_ERR_LOGFILE,		"PM_ERR_LOGFILE",
-	"Missing PCP archive log file" },
+	"Missing PCP archive file" },
     { PM_ERR_NOTARCHIVE,	"PM_ERR_NOTARCHIVE",
 	"Operation requires context with archive source of metrics" },
     { PM_ERR_NOCONTEXT,		"PM_ERR_NOCONTEXT",
@@ -131,11 +131,11 @@ static const struct {
     { PM_ERR_PROFILESPEC,	"PM_ERR_PROFILESPEC",
 	"NULL pmInDom with non-NULL instlist" },
     { PM_ERR_PMID_LOG,		"PM_ERR_PMID_LOG",
-	"Metric not defined in the PCP archive log" },
+	"Metric not defined in the PCP archive" },
     { PM_ERR_INDOM_LOG,		"PM_ERR_INDOM_LOG",
-	"Instance domain identifier not defined in the PCP archive log" },
+	"Instance domain identifier not defined in the PCP archive" },
     { PM_ERR_INST_LOG,		"PM_ERR_INST_LOG",
-	"Instance identifier not defined in the PCP archive log" },
+	"Instance identifier not defined in the PCP archive" },
     { PM_ERR_NOPROFILE,		"PM_ERR_NOPROFILE",
 	"Missing profile - protocol botch" },
     { PM_ERR_NOAGENT,		"PM_ERR_NOAGENT",
@@ -219,8 +219,7 @@ strerror_x(int code, char *buf, int buflen)
     char	*p;
     p = strerror_r(code, buf, buflen);
     if (p != buf) {
-	strncpy(buf, p, buflen);
-	buf[buflen-1] = '\0';
+	pmstrncpy(buf, buflen, p);
     }
 #else
     /*
@@ -243,8 +242,7 @@ pmErrStr_r(int code, char *buf, int buflen)
 #endif
 
     if (code == 0) {
-	strncpy(buf, "No error", buflen);
-	buf[buflen-1] = '\0';
+	pmstrncpy(buf, buflen, "No error");
 	return buf;
     }
 
@@ -328,16 +326,14 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_ALLOC);
     if (code > -PM_ERR_BASE || code < -PM_ERR_NYI) {
 	const char	*bp;
 	if ((bp = wsastrerror(-code)) != NULL) {
-	    strncpy(buf, bp, buflen);
-	    buf[buflen-1] = '\0';
+	    pmstrncpy(buf, buflen, bp);
 	}
 	else {
 	    /* No strerror_r in MinGW, so need to lock */
 	    char	*tbp;
 	    PM_LOCK(__pmLock_extcall);
 	    tbp = strerror(-code);		/* THREADSAFE */
-	    strncpy(buf, tbp, buflen);
-	    buf[buflen-1] = '\0';
+	    pmstrncpy(buf, buflen, tbp);
 	    PM_UNLOCK(__pmLock_extcall);
 	}
 
@@ -348,8 +344,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_ALLOC);
 
     for (i = 0; errtab[i].err; i++) {
 	if (errtab[i].err == code) {
-	    strncpy(buf, errtab[i].errmess, buflen);
-	    buf[buflen-1] = '\0';
+	    pmstrncpy(buf, buflen, errtab[i].errmess);
 	    return buf;
 	}
     }
