@@ -358,7 +358,7 @@ hashsummary(char *where, __pmHashCtl *hcp)
     int			nnode = 0;
     __pmHashNode	*hp;
 
-    fprintf(stderr, "%s(): hcp=%p", where, hcp);
+    fprintf(stderr, "%s(): hcp=" PRINTF_P_PFX "%p", where, hcp);
     if (hcp != NULL) {
 	fprintf(stderr, " nodes=%d hsize=%d", hcp->nodes, hcp->hsize);
 	for (i = 0; i < hcp->hsize; i++) {
@@ -369,7 +369,7 @@ hashsummary(char *where, __pmHashCtl *hcp)
 		    nnode++;
 	    }
 	}
-	fprintf(stderr, " empty_slots=%d nodes=%d next=%p index=%d",
+	fprintf(stderr, " empty_slots=%d nodes=%d next=" PRINTF_P_PFX "%p index=%d",
 		nempty, nnode, hcp->next, hcp->index);
 	if (nnode != hcp->nodes)
 	    fprintf(stderr, "\nBotch! found nodes %d != hash nodes %d", nnode, hcp->nodes);
@@ -466,7 +466,7 @@ logFreeHashInDom(__pmHashCtl *hcp)
 	    for (idp = (__pmLogInDom *)hp->data, prior_idp = NULL;
 		idp != NULL; idp = idp->next) {
 		if ((idp->alloc & ~(PMLID_SELF|PMLID_INSTLIST|PMLID_NAMELIST|PMLID_NAMES)) != 0) {
-		    fprintf(stderr, "logFreeHashInDom(%p): Warning: bogus alloc flags: 0x%x for idp=%p\n",
+		    fprintf(stderr, "logFreeHashInDom(" PRINTF_P_PFX "%p): Warning: bogus alloc flags: 0x%x for idp=" PRINTF_P_PFX "%p\n",
 			hcp, idp->alloc & ~(PMLID_SELF|PMLID_INSTLIST|PMLID_NAMELIST|PMLID_NAMES), idp);
 		}
 
@@ -1123,12 +1123,12 @@ __pmLogResetCB(const __pmArchCtl *acp, int volume, long offset, const char *call
     __pmFseek(fp, offset, SEEK_SET);
 }
 
-static long
+static off_t
 __pmLogTellCB(const __pmArchCtl *acp, int volume, const char *caller)
 {
     const char		*id;
     __pmFILE		*fp;
-    long		off;
+    off_t		off;
 
     fp = logCallBackFile(acp, volume, &id);
 
@@ -1222,7 +1222,7 @@ logputresult(int version, __pmArchCtl *acp, __pmPDU *pb, const char *caller)
 
     if (pmDebugOptions.log)
 	fprintf(stderr, "%s: pdubuf=" PRINTF_P_PFX "%p"
-		" input len=%d output len=%d posn=%ld\n", __FUNCTION__,
+		" input len=%d output len=%d posn=%" FMT_INT64 "\n", __FUNCTION__,
 		pb, pb[0], sz, acp->ac_tell_cb(acp, PM_LOG_VOL_CURRENT, __FUNCTION__));
 
     save_from = start[0];
