@@ -110,6 +110,7 @@ __pmDecodeLogControl(const __pmPDU *pdubuf, __pmResult **request, int *control,
     const control_req_t	*pp;
     char		*pduend;
     int			numpmid;
+    int			maxnumpmid;
     size_t		need;
     __pmResult		*req;
     pmValueSet		*vsp;
@@ -117,10 +118,14 @@ __pmDecodeLogControl(const __pmPDU *pdubuf, __pmResult **request, int *control,
 
     pp = (const control_req_t *)pdubuf;
     pduend = (char *)pdubuf + pp->c_hdr.len;
-    if (pduend - (char *)pdubuf < sizeof(control_req_t)) {
+    /*
+     * c_data has at least on pmid, so a minimal 2 words for
+     * v_pmid and v_numval, less the 1 word for c_data
+     */
+    if (pp->c_hdr.len < sizeof(control_req_t) + sizeof(__pmPDU)) {
 	if (pmDebugOptions.pdu) {
-	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: remainder %d < sizeof(control_req_t) %d\n",
-		(int)(pduend - (char*)pdubuf), (int)sizeof(control_req_t));
+	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: short PDU %d < min size %d\n",
+		pp->c_hdr.len, (int)(sizeof(control_req_t) + sizeof(__pmPDU)));
 	}
 	return PM_ERR_IPC;
     }
@@ -131,20 +136,22 @@ __pmDecodeLogControl(const __pmPDU *pdubuf, __pmResult **request, int *control,
     numpmid = ntohl(pp->c_numpmid);
     vp = (vlist_t *)pp->c_data;
 
-    if (numpmid < 0 || numpmid > pp->c_hdr.len) {
+    if (numpmid < 0) {
 	if (pmDebugOptions.pdu) {
-	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: numpmid %d < 0 or > c_hdr.len %d\n",
-		numpmid, pp->c_hdr.len);
+	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: numpmid %d < 0\n",
+		numpmid);
 	}
 	return PM_ERR_IPC;
     }
-    if (numpmid >= (INT_MAX - sizeof(pmResult)) / sizeof(pmValueSet *)) {
+    /* v_numval may be 0, so v_vlist[] not there => min 2 words per pmid */
+    maxnumpmid = (int)((pp->c_hdr.len - sizeof(control_req_t) + sizeof(__pmPDU)) / (2 * sizeof(__pmPDU)));
+    if (numpmid > maxnumpmid) {
 	if (pmDebugOptions.pdu) {
-	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: numpmid %d >= (INT_MAX ...) %d\n",
-		numpmid, (int)((INT_MAX - sizeof(pmResult)) / sizeof(pmValueSet *)));
+	    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: numpmid %d > max %d for PDU len %d\n",
+		numpmid, maxnumpmid, pp->c_hdr.len);
 	}
 	return PM_ERR_IPC;
-    }
+	}
     if ((req = __pmAllocResult(numpmid)) == NULL) {
 	pmNoMem("__pmDecodeLogControl.req", sizeof(__pmResult) + (numpmid - 1) * sizeof(pmValueSet *), PM_RECOV_ERR);
 	return -oserror();
@@ -173,19 +180,12 @@ __pmDecodeLogControl(const __pmPDU *pdubuf, __pmResult **request, int *control,
 	    }
 	} else {
 	    /* check that dynamic allocation argument will not wrap */
-	    if (nv >= (INT_MAX - sizeof(pmValueSet)) / sizeof(pmValue)) {
-		if (pmDebugOptions.pdu) {
-		    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: dst nv %d < (INT_MAX ...) %d\n",
-			nv, (int)((INT_MAX - sizeof(pmValueSet)) / sizeof(pmValue)));
-		}
-		goto corrupt;
-	    }
 	    need = sizeof(pmValueSet) + ((nv - 1) * sizeof(pmValue));
 	    /* check that pointer cannot move beyond input buffer end */
-	    if (nv >= (INT_MAX - sizeof(vlist_t)) / sizeof(__pmValue_PDU)) {
+	    if (nv >= (pp->c_hdr.len - sizeof(vlist_t)) / sizeof(__pmValue_PDU)) {
 		if (pmDebugOptions.pdu) {
-		    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: src nv %d < (INT_MAX ...) %d\n",
-			nv, (int)((INT_MAX - sizeof(vlist_t)) / sizeof(__pmValue_PDU)));
+		    fprintf(stderr, "__pmDecodeLogControl: PM_ERR_IPC: pmid[%d] numval %d > max %d for PDU len %d\n",
+			i, nv, (int)((pp->c_hdr.len - sizeof(vlist_t)) / sizeof(__pmValue_PDU)), pp->c_hdr.len);
 		}
 		goto corrupt;
 	    }
