@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2014-2015,2018 Red Hat.
+ * Copyright (c) 2014-2015,2018,2022 Red Hat.
  * Copyright (c) 1995 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
@@ -63,12 +63,16 @@ static const struct {
     int		len;		/* length of scale name */
     int		scale;		/* <0 -divisor, else multiplier */
 } int_tab[] = {
+    { "nanosecond",    10, -1000000000 }, 
+    { "microsecond",   11, -1000000 }, 
     { "millisecond",   11, -1000 }, 
     { "second",		6,     1 },
     { "minute",		6,    60 },
     { "hour",		4,  3600 },
     { "day",		3, 86400 },
     { "msec",		4, -1000 },
+    { "usec",		4, -1000000 },
+    { "nsec",		4, -1000000000 },
     { "sec",		3,     1 },
     { "min",		3,    60 },
     { "hr",		2,  3600 },
@@ -84,17 +88,17 @@ static const int	numint = sizeof(int_tab) / sizeof(int_tab[0]);
 #define NEG_OFFSET	2
 
 
-/* Compare struct timevals */
-static int	/* 0 -> equal, -1 -> tv1 < tv2, 1 -> tv1 > tv2 */
-tvcmp(struct timeval tv1, struct timeval tv2)
+/* Compare struct timespecs */
+static int	/* 0 -> equal, -1 -> t1 < t2, 1 -> t1 > t2 */
+tscmp(struct timespec t1, struct timespec t2)
 {
-    if (tv1.tv_sec < tv2.tv_sec)
+    if (t1.tv_sec < t2.tv_sec)
 	return -1;
-    if (tv1.tv_sec > tv2.tv_sec)
+    if (t1.tv_sec > t2.tv_sec)
 	return 1;
-    if (tv1.tv_usec < tv2.tv_usec)
+    if (t1.tv_nsec < t2.tv_nsec)
 	return -1;
-    if (tv1.tv_usec > tv2.tv_usec)
+    if (t1.tv_nsec > t2.tv_nsec)
 	return 1;
     return 0;
 }
@@ -200,7 +204,7 @@ parseError(const char *spec, const char *point, char *msg, char **rslt)
     char	*q;
 
     if ((*rslt = malloc(need)) == NULL)
-	pmNoMem("__pmParseTime", need, PM_FATAL_ERR);
+	pmNoMem("__pmtimevalParse", need, PM_FATAL_ERR);
     q = *rslt;
 
     for (p = spec; *p != '\0'; p++)
@@ -217,10 +221,10 @@ parseError(const char *spec, const char *point, char *msg, char **rslt)
 }
 
 
-int		/* 0 -> ok, -1 -> error */
-pmParseInterval(
+static int	/* 0 -> ok, -1 -> error */
+__pmParseInterval(
     const char *spec,		/* interval to parse */
-    struct timeval *rslt,	/* result stored here */
+    double *rslt,		/* result stored here */
     char **errmsg)		/* error message */
 {
     const char	*scan = spec;
@@ -238,7 +242,7 @@ pmParseInterval(
 
     /* parse components of spec */
     while (*scan != '\0') {
-	if (! parseDouble(&scan, 0.0, (double)INT_MAX, &d))
+	if (! parseDouble(&scan, 0.0, (double)PM_MAX_TIME_T, &d))
 	    break;
 	while (*scan != '\0' && isspace((int)*scan))
 	    scan++;
@@ -279,11 +283,41 @@ pmParseInterval(
 	return -1;
     }
 
-    /* convert into seconds and microseconds */
-    pmtimevalFromReal(sec, rslt);
+    *rslt = sec;
     return 0;
 }
 
+int		/* 0 -> ok, -1 -> error */
+pmParseInterval_v2(
+    const char *spec,		/* interval to parse */
+    struct timeval *rslt,	/* result stored here */
+    char **errmsg)		/* error message */
+{
+    double	secs;
+    int		sts;
+
+    if ((sts = __pmParseInterval(spec, &secs, errmsg)) < 0)
+	return sts;
+    /* convert into seconds and microseconds */
+    pmtimevalFromReal(secs, rslt);
+    return 0;
+}
+
+int		/* 0 -> ok, -1 -> error */
+pmParseInterval(
+    const char *spec,		/* interval to parse */
+    struct timespec *rslt,	/* result stored here */
+    char **errmsg)		/* error message */
+{
+    double	secs;
+    int		sts;
+
+    if ((sts = __pmParseInterval(spec, &secs, errmsg)) < 0)
+	return sts;
+    /* convert into seconds and nanoseconds */
+    pmtimespecFromReal(secs, rslt);
+    return 0;
+}
 
 int		/* 0 -> ok, -1 -> error */
 __pmParseCtime(
@@ -322,8 +356,8 @@ __pmParseCtime(
 	parseInt(&scan, 0, 59, &tm.tm_min);
 	if (parseChar(&scan, ':')) {
 	    if (parseDouble(&scan, 0.0, 61.0, &d)) {
-		tm.tm_sec = (time_t)d;
-		tm.tm_yday = (int)((long double)1000000 * (long double)(d - tm.tm_sec));
+		tm.tm_sec = (int)d;	/* tm_sec is in range 0-60 */
+		tm.tm_yday = (int)((long double)1000000000 * (long double)(d - tm.tm_sec));
 	    }
 	}
     }
@@ -395,33 +429,57 @@ __pmParseCtime(
 
 
 int	/* 0 ok, -1 error */
-__pmConvertTime(
+__pmtimevalConvert(
     struct tm *tmin,		/* absolute or +ve or -ve offset time */
-    struct timeval *origin,	/* defaults and origin for offset */
+    struct timeval *orig,	/* defaults and origin for offset */
     struct timeval *rslt)	/* result stored here */
+{
+    struct timespec origin, result;
+    int sts;
+
+    origin.tv_sec = orig->tv_sec;
+    if (orig->tv_usec == 999999)
+	origin.tv_nsec = 999999999;
+    else
+	origin.tv_nsec = orig->tv_usec * 1000;
+
+    if ((sts = __pmtimespecConvert(tmin, &origin, &result)) < 0)
+	return sts;
+
+    rslt->tv_sec = result.tv_sec;
+    rslt->tv_usec = result.tv_nsec / 1000;
+    return sts;
+}
+
+
+int	/* 0 ok, -1 error */
+__pmtimespecConvert(
+    struct tm *tmin,		/* absolute or +ve or -ve offset time */
+    struct timespec *origin,	/* defaults and origin for offset */
+    struct timespec *result)	/* result stored here */
 {
     time_t	    t;
-    struct timeval  tval = *origin;
+    struct timespec tspec = *origin;
     struct tm	    tm;
 
     /* positive offset interval */
     if (tmin->tm_wday == PLUS_OFFSET) {
-	tval.tv_usec += tmin->tm_yday;
-	if (tval.tv_usec > 1000000) {
-	    tval.tv_usec -= 1000000;
-	    tval.tv_sec++;
+	tspec.tv_nsec += tmin->tm_yday;
+	if (tspec.tv_nsec > 1000000000) {
+	    tspec.tv_nsec -= 1000000000;
+	    tspec.tv_sec++;
 	}
-	tval.tv_sec += tmin->tm_sec;
+	tspec.tv_sec += tmin->tm_sec;
     }
 
     /* negative offset interval */
     else if (tmin->tm_wday == NEG_OFFSET) {
-	if (tval.tv_usec < tmin->tm_yday) {
-	    tval.tv_usec += 1000000;
-	    tval.tv_sec--;
+	if (tspec.tv_nsec < tmin->tm_yday) {
+	    tspec.tv_nsec += 1000000000;
+	    tspec.tv_sec--;
 	}
-	tval.tv_usec -= tmin->tm_yday;
-	tval.tv_sec -= tmin->tm_sec;
+	tspec.tv_nsec -= tmin->tm_yday;
+	tspec.tv_sec -= tmin->tm_sec;
     }
 
     /* absolute time */
@@ -429,12 +487,12 @@ __pmConvertTime(
 	/* tmin completely specified */
 	if (tmin->tm_year != -1) {
 	    tm = *tmin;
-	    tval.tv_usec = tmin->tm_yday;
+	    tspec.tv_nsec = tmin->tm_yday;
 	}
 
 	/* tmin partially specified */
 	else {
-	    t = (time_t)tval.tv_sec;
+	    t = tspec.tv_sec;
 	    pmLocaltime(&t, &tm);
 	    tm.tm_isdst = -1;
 
@@ -452,7 +510,7 @@ __pmConvertTime(
 		tm.tm_hour = tmin->tm_hour;
 		tm.tm_min = tmin->tm_min;
 		tm.tm_sec = tmin->tm_sec;
-		tval.tv_usec = tmin->tm_yday;
+		tspec.tv_nsec = tmin->tm_yday;
 	    }
 	    else if (tmin->tm_mday != -1) {
 		if (tmin->tm_mday < tm.tm_mday)
@@ -461,7 +519,7 @@ __pmConvertTime(
 		tm.tm_hour = tmin->tm_hour;
 		tm.tm_min = tmin->tm_min;
 		tm.tm_sec = tmin->tm_sec;
-		tval.tv_usec = tmin->tm_yday;
+		tspec.tv_nsec = tmin->tm_yday;
 	    }
 	    else if (tmin->tm_hour != -1) {
 		if (tmin->tm_hour < tm.tm_hour)
@@ -469,26 +527,26 @@ __pmConvertTime(
 		tm.tm_hour = tmin->tm_hour;
 		tm.tm_min = tmin->tm_min;
 		tm.tm_sec = tmin->tm_sec;
-		tval.tv_usec = tmin->tm_yday;
+		tspec.tv_nsec = tmin->tm_yday;
 	    }
 	    else if (tmin->tm_min != -1) {
 		if (tmin->tm_min < tm.tm_min)
 		    tm.tm_hour++;
 		tm.tm_min = tmin->tm_min;
 		tm.tm_sec = tmin->tm_sec;
-		tval.tv_usec = tmin->tm_yday;
+		tspec.tv_nsec = tmin->tm_yday;
 	    }
 	    else if (tmin->tm_sec != -1) {
 		if (tmin->tm_sec < tm.tm_sec)
 		    tm.tm_min++;
 		tm.tm_sec = tmin->tm_sec;
-		tval.tv_usec = tmin->tm_yday;
+		tspec.tv_nsec = tmin->tm_yday;
 	    }
 	}
-	tval.tv_sec = __pmMktime(&tm);
+	tspec.tv_sec = __pmMktime(&tm);
     }
 
-    *rslt = tval;
+    *result = tspec;
     return 0;
 }
 
@@ -547,76 +605,60 @@ glib_relative_date(const char *date_string)
 static int
 glib_get_date(
     const char		*scan,
-    struct timeval	*start,
-    struct timeval	*end,
-    struct timeval	*rslt)
+    struct timespec	*start,
+    struct timespec	*end,
+    struct timespec	*rslt)
 {
-    int sts;
     int rel_type;
-    struct timespec tsrslt;
 
     rel_type = glib_relative_date(scan);
 
     if (rel_type == NO_OFFSET)
-	sts = __pmGetDate(&tsrslt, scan, NULL);
-    else if (rel_type == NEG_OFFSET && end->tv_sec < INT_MAX) {
-	struct timespec tsend;
-	tsend.tv_sec = end->tv_sec;
-	tsend.tv_nsec = end->tv_usec * 1000;
-	sts = __pmGetDate(&tsrslt, scan, &tsend);
-    }
-    else {
-	struct timespec tsstart;
-	tsstart.tv_sec = start->tv_sec;
-	tsstart.tv_nsec = start->tv_usec * 1000;
-	sts = __pmGetDate(&tsrslt, scan, &tsstart);
-    }
-    if (sts < 0)
-	return sts;
-
-    rslt->tv_sec = tsrslt.tv_sec;
-    rslt->tv_usec = tsrslt.tv_nsec / 1000;
-    return 0;
+	return __pmGetDate(rslt, scan, NULL);
+    else if (rel_type == NEG_OFFSET && end->tv_sec < PM_MAX_TIME_T)
+	return __pmGetDate(rslt, scan, end);
+    else
+	return __pmGetDate(rslt, scan, start);
 }
 
 int	/* 0 -> ok, -1 -> error */
-__pmParseTime(
+__pmtimespecParse(
     const char	    *string,	/* string to be parsed */
-    struct timeval  *logStart,	/* start of log or current time */
-    struct timeval  *logEnd,	/* end of log or tv_sec == INT_MAX */
-				/* assumes sizeof(t_time) == sizeof(int) */
-    struct timeval  *rslt,	/* if parsed ok, result filled in */
+    struct timespec *logStart,	/* start of log or current time */
+    struct timespec *logEnd,	/* end of log or tv_sec == LONG_MAX */
+				/* assumes sizeof(time_t) == sizeof(long) */
+    struct timespec *rslt,	/* if parsed ok, result filled in */
     char	    **errMsg)	/* error message, please free */
 {
     struct tm	    tm;
     const char	    *scan;
-    struct timeval  start;
-    struct timeval  end;
-    struct timeval  tval;
+    struct timespec start;
+    struct timespec end;
+    struct timespec tspec;
 
     *errMsg = NULL;
     start = *logStart;
     end = *logEnd;
-    if (end.tv_sec == INT_MAX)
-	end.tv_usec = 999999;
+    if (end.tv_sec == PM_MAX_TIME_T)
+	end.tv_nsec = 999999999;
     scan = string;
 
     /* ctime string */
     if (parseChar(&scan, '@')) {
 	if (__pmParseCtime(scan, &tm, errMsg) >= 0) {
 	    tm.tm_wday = NO_OFFSET;
-	    __pmConvertTime(&tm, &start, rslt);
+	    __pmtimespecConvert(&tm, &start, rslt);
 	    return 0;
 	}
     }
 
     /* relative to end of archive */
-    else if (end.tv_sec < INT_MAX && parseChar(&scan, '-')) {
-	if (pmParseInterval(scan, &tval, errMsg) >= 0) {
+    else if (end.tv_sec < PM_MAX_TIME_T && parseChar(&scan, '-')) {
+	if (pmParseInterval(scan, &tspec, errMsg) >= 0) {
 	    tm.tm_wday = NEG_OFFSET;
-	    tm.tm_sec = (int)tval.tv_sec;
-	    tm.tm_yday = (int)tval.tv_usec;
-	    __pmConvertTime(&tm, &end, rslt);
+	    tm.tm_sec = tspec.tv_sec;
+	    tm.tm_yday = tspec.tv_nsec;
+	    __pmtimespecConvert(&tm, &end, rslt);
 	    return 0;
 	}
     }
@@ -624,11 +666,11 @@ __pmParseTime(
     /* relative to start of archive or current time */
     else {
 	parseChar(&scan, '+');
-	if (pmParseInterval(scan, &tval, errMsg) >= 0) {
+	if (pmParseInterval(scan, &tspec, errMsg) >= 0) {
 	    tm.tm_wday = PLUS_OFFSET;
-	    tm.tm_sec = (int)tval.tv_sec;
-	    tm.tm_yday = (int)tval.tv_usec;
-	    __pmConvertTime(&tm,  &start, rslt);
+	    tm.tm_sec = tspec.tv_sec;
+	    tm.tm_yday = tspec.tv_nsec;
+	    __pmtimespecConvert(&tm, &start, rslt);
 	    return 0;
 	}
     }
@@ -636,8 +678,7 @@ __pmParseTime(
     /*
      * if we get here, *errMsg is not NULL, because one of
      * - __pmParseCtime(), or
-     * - pmParseInterval(), or
-     * - the other pmParseInterval()
+     * - pmParseInterval()
      * returned a value < 0 ... if glib_get_date() fails we're
      * going to return with the previously set *errMsg
      */
@@ -659,29 +700,93 @@ __pmParseTime(
     return 0;
 }
 
+int	/* 0 -> ok, -1 -> error */
+__pmtimevalParse(
+    const char	    *string,	/* string to be parsed */
+    struct timeval  *logStart,	/* start of log or current time */
+    struct timeval  *logEnd,	/* end of log or tv_sec == PM_MAX_TIME_T */
+    struct timeval  *rslt,	/* if parsed ok, result filled in */
+    char	    **errMsg)	/* error message, please free */
+{
+    struct timespec start, end, result;
+    int		    sts;
 
-/* This function is designed to encapsulate the interpretation of
-   the -S, -T, -A and -O command line switches for use by the PCP
-   client tools. */
+    start.tv_sec = logStart->tv_sec;
+    start.tv_nsec = logStart->tv_usec * 1000;
+    end.tv_sec = logEnd->tv_sec;
+    end.tv_nsec = logEnd->tv_usec * 1000;
+
+    if ((sts = __pmtimespecParse(string, &start, &end, &result, errMsg)) < 0)
+	return sts;
+
+    rslt->tv_sec = result.tv_sec;
+    rslt->tv_usec = result.tv_nsec / 1000;
+
+    return sts;
+}
+
+int    /* 1 -> ok, 0 -> warning, -1 -> error */
+pmParseTimeWindow_v2(
+    const char      *swStart,   /* argument of -S switch, may be NULL */
+    const char      *swEnd,     /* argument of -T switch, may be NULL */
+    const char      *swAlign,   /* argument of -A switch, may be NULL */
+    const char	    *swOffset,	/* argument of -O switch, may be NULL */
+    const struct timeval  *logStart,  /* start of log or current time */
+    const struct timeval  *logEnd,    /* end of log or tv_sec == PM_MAX_TIME_T */
+    struct timeval  *rsltStart, /* start time returned here */
+    struct timeval  *rsltEnd,   /* end time returned here */
+    struct timeval  *rsltOffset,/* offset time returned here */
+    char            **errMsg)	/* error message, please free */
+{
+    struct timespec logstart;
+    struct timespec logend;
+    struct timespec start;
+    struct timespec end;
+    struct timespec offset;
+    int		    sts;
+
+    logstart.tv_sec = logStart->tv_sec;
+    logstart.tv_nsec = logStart->tv_usec * 1000;
+    logend.tv_sec = logEnd->tv_sec;
+    logend.tv_nsec = logEnd->tv_usec * 1000;
+
+    if ((sts = pmParseTimeWindow(swStart, swEnd, swAlign, swOffset,
+			&logstart, &logend, &start, &end, &offset, errMsg)) < 0)
+	return sts;
+
+    rsltStart->tv_sec = start.tv_sec;
+    rsltStart->tv_usec = start.tv_nsec / 1000;
+    rsltEnd->tv_sec = end.tv_sec;
+    rsltEnd->tv_usec = end.tv_nsec / 1000;
+    rsltOffset->tv_sec = offset.tv_sec;
+    rsltOffset->tv_usec = offset.tv_nsec / 1000;
+    return sts;
+}
+
+/*
+ * This function is designed to encapsulate the interpretation of
+ * the -S, -T, -A and -O command line switches for use by the PCP
+ * client tools.
+ */
 int    /* 1 -> ok, 0 -> warning, -1 -> error */
 pmParseTimeWindow(
     const char      *swStart,   /* argument of -S switch, may be NULL */
     const char      *swEnd,     /* argument of -T switch, may be NULL */
     const char      *swAlign,   /* argument of -A switch, may be NULL */
     const char	    *swOffset,	/* argument of -O switch, may be NULL */
-    const struct timeval  *logStart,  /* start of log or current time */
-    const struct timeval  *logEnd,    /* end of log or tv_sec == INT_MAX */
-    struct timeval  *rsltStart, /* start time returned here */
-    struct timeval  *rsltEnd,   /* end time returned here */
-    struct timeval  *rsltOffset,/* offset time returned here */
+    const struct timespec *logStart,  /* start of log or current time */
+    const struct timespec *logEnd,    /* end of log or tv_sec == PM_MAX_TIME_T */
+    struct timespec *rsltStart, /* start time returned here */
+    struct timespec *rsltEnd,   /* end time returned here */
+    struct timespec *rsltOffset,/* offset time returned here */
     char            **errMsg)	/* error message, please free */
 {
-    struct timeval  astart;
-    struct timeval  start;
-    struct timeval  end;
-    struct timeval  offset;
-    struct timeval  aoffset;
-    struct timeval  tval;
+    struct timespec astart;
+    struct timespec start;
+    struct timespec end;
+    struct timespec offset;
+    struct timespec aoffset;
+    struct timespec tspec;
     const char	    *scan;
     __int64_t	    delta = 0;	/* initialize to pander to gcc */
     __int64_t 	    align;
@@ -691,40 +796,74 @@ pmParseTimeWindow(
     /* default values for start and end */
     start = *logStart;
     end = *logEnd;
-    if (end.tv_sec == INT_MAX)
-	end.tv_usec = 999999;
+    if (end.tv_sec == PM_MAX_TIME_T)
+	end.tv_nsec = 999999999;
 
     /* parse -S argument and adjust start accordingly */
     if (swStart) {
-	if (__pmParseTime(swStart, &start, &end, &start, errMsg) < 0)
+	if (__pmtimespecParse(swStart, &start, &end, &start, errMsg) < 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -S %s => %s\n", swStart, *errMsg);
+	    }
 	    return -1;
+	}
+	else if (pmDebugOptions.getopt) {
+	    fprintf(stderr, "pmParseTimeWindow: -S %s => logstart=", swStart);
+	    pmtimespecPrint(stderr, logStart);
+	    fprintf(stderr, " start=");
+	    pmtimespecPrint(stderr, &start);
+	    fprintf(stderr, " logend=");
+	    pmtimespecPrint(stderr, logEnd);
+	    fputc('\n', stderr);
+	}
     }
 
     /* sanity check -S */
-    if (tvcmp(start, *logStart) < 0)
+    if (tscmp(start, *logStart) < 0) {
 	/* move start forwards to the beginning of the archive */
+	if (pmDebugOptions.getopt) {
+	    fprintf(stderr, "pmParseTimeWindow: -S %s start ", swStart);
+	    pmtimespecPrint(stderr, &start);
+	    fprintf(stderr, " before logstart ");
+	    pmtimespecPrint(stderr, logStart);
+	    fprintf(stderr, " advance to logstart\n");
+	}
 	start = *logStart;
+    }
 
     /* parse -A argument and adjust start accordingly */
     if (swAlign) {
 	scan = swAlign;
-	if (pmParseInterval(scan, &tval, errMsg) < 0)
+	if (pmParseInterval(scan, &tspec, errMsg) < 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -A %s => %s\n", swAlign, *errMsg);
+	    }
 	    return -1;
-	if (tval.tv_sec == 0 && tval.tv_usec == 0) {
+	}
+	if (tspec.tv_sec == 0 && tspec.tv_nsec == 0) {
 	    parseError(swAlign, swAlign, alignmsg, errMsg);
 	    return -1;
 	}
-	delta = tval.tv_usec + 1000000 * (__int64_t)tval.tv_sec;
-	align = start.tv_usec + 1000000 * (__int64_t)start.tv_sec;
+	delta = tspec.tv_nsec + 1000000000 * (__int64_t)tspec.tv_sec;
+	align = start.tv_nsec + 1000000000 * (__int64_t)start.tv_sec;
 	blign = (align / delta) * delta;
 	if (blign < align)
 	    blign += delta;
-	astart.tv_sec = (time_t)(blign / 1000000);
-	astart.tv_usec = (int)(blign % 1000000);
+	astart.tv_sec = (time_t)(blign / 1000000000);
+	astart.tv_nsec = (int)(blign % 1000000000);
 
 	/* sanity check -S after alignment */
-	if (tvcmp(astart, *logStart) >= 0 && tvcmp(astart, *logEnd) <= 0)
+	if (tscmp(astart, *logStart) >= 0 && tscmp(astart, *logEnd) <= 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -A %s", swAlign);
+		fprintf(stderr, " advance start from ");
+		pmtimespecPrint(stderr, &start);
+		fprintf(stderr, " to ");
+		pmtimespecPrint(stderr, &astart);
+		fputc('\n', stderr);
+	    }
 	    start = astart;
+	}
 	else {
 	    parseError(swAlign, swAlign, alignmsg, errMsg);
 	    sts = 0;
@@ -733,41 +872,95 @@ pmParseTimeWindow(
 
     /* parse -T argument and adjust end accordingly */
     if (swEnd) {
-	if (__pmParseTime(swEnd, &start, &end, &end, errMsg) < 0)
+	if (__pmtimespecParse(swEnd, &start, &end, &end, errMsg) < 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -T %s => %s\n", swEnd, *errMsg);
+	    }
 	    return -1;
+	}
+	else if (pmDebugOptions.getopt) {
+	    fprintf(stderr, "pmParseTimeWindow: -T %s => logstart=", swStart);
+	    pmtimespecPrint(stderr, logStart);
+	    fprintf(stderr, " end=");
+	    pmtimespecPrint(stderr, &end);
+	    fprintf(stderr, " logend=");
+	    pmtimespecPrint(stderr, logEnd);
+	    fputc('\n', stderr);
+	}
     }
 
     /* sanity check -T */
-    if (tvcmp(end, *logEnd) > 0)
+    if (tscmp(end, *logEnd) > 0) {
 	/* move end backwards to the end of the archive */
+	if (pmDebugOptions.getopt) {
+	    fprintf(stderr, "pmParseTimeWindow: -T %s start ", swEnd);
+	    pmtimespecPrint(stderr, &end);
+	    fprintf(stderr, " after logend ");
+	    pmtimespecPrint(stderr, logEnd);
+	    fprintf(stderr, " trim to logend\n");
+	}
 	end = *logEnd;
+    }
 
     /* parse -O argument and align if required */
     offset = start;
     if (swOffset) {
-	if (__pmParseTime(swOffset, &start, &end, &offset, errMsg) < 0)
+	if (__pmtimespecParse(swOffset, &start, &end, &offset, errMsg) < 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -O %s => %s\n", swOffset, *errMsg);
+	    }
 	    return -1;
+	}
 
 	/* sanity check -O */
-	if (tvcmp(offset, start) < 0)
+	if (tscmp(offset, start) < 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -O %s offset ", swOffset);
+		pmtimespecPrint(stderr, &offset);
+		fprintf(stderr, " before start ");
+		pmtimespecPrint(stderr, &start);
+		fprintf(stderr, " advance to start\n");
+	    }
 	    offset = start;
-	else if (tvcmp(offset, end) > 0)
+	}
+	else if (tscmp(offset, end) > 0) {
+	    if (pmDebugOptions.getopt) {
+		fprintf(stderr, "pmParseTimeWindow: -O %s offset ", swOffset);
+		pmtimespecPrint(stderr, &offset);
+		fprintf(stderr, " after end ");
+		pmtimespecPrint(stderr, &end);
+		fprintf(stderr, " trim to end\n");
+	    }
 	    offset = end;
+	}
 
-	if (swAlign) {
-	    align = offset.tv_usec + 1000000 * (__int64_t)offset.tv_sec;
-	    blign = (align / delta) * delta;
+	/*
+	 * if end.tv_sec == PM_MAX_TIME_T then we cannot do -A rounding
+	 * due to arithmetic overflow [Coverity CID 464604]
+	 */
+	if (swAlign && end.tv_sec != PM_MAX_TIME_T) {
+	    align = offset.tv_nsec + 1000000000 *
+	    (__int64_t)offset.tv_sec; blign = (align / delta) * delta;
 	    if (blign < align)
 		blign += delta;
-	    align = end.tv_usec + 1000000 * (__int64_t)end.tv_sec;
+	    align = end.tv_nsec + 1000000000 * (__int64_t)end.tv_sec;
 	    if (blign > align)
 		blign -= delta;
-	    aoffset.tv_sec = (time_t)(blign / 1000000);
-	    aoffset.tv_usec = (int)(blign % 1000000);
+	    aoffset.tv_sec = (time_t)(blign / 1000000000);
+	    aoffset.tv_nsec = (int)(blign % 1000000000);
 
 	    /* sanity check -O after alignment */
-	    if (tvcmp(aoffset, start) >= 0 && tvcmp(aoffset, end) <= 0)
+	    if (tscmp(aoffset, start) >= 0 && tscmp(aoffset, end) <= 0) {
+		if (pmDebugOptions.getopt) {
+		    fprintf(stderr, "pmParseTimeWindow: -A %s", swAlign);
+		    fprintf(stderr, " advance offset from ");
+		    pmtimespecPrint(stderr, &offset);
+		    fprintf(stderr, " to ");
+		    pmtimespecPrint(stderr, &aoffset);
+		    fputc('\n', stderr);
+		}
 		offset = aoffset;
+	    }
 	    else {
 		parseError(swAlign, swAlign, alignmsg, errMsg);
 		sts = 0;
