@@ -20,8 +20,10 @@
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
+#include <valkey/alloc.h>
+#include "adlist.h"
 #if defined(HAVE_LIBUV)
-#include <hiredis-cluster/adapters/libuv.h>
+#include <valkey/adapters/libuv.h>
 #else
 static int keyClusterLibuvAttach() { return RESP_OK; }
 #endif
@@ -29,7 +31,7 @@ static int keyClusterLibuvAttach() { return RESP_OK; }
 static char default_server[] = "localhost:6379";
 
 static void
-key_server_connect_callback(const keysAsyncContext *keys, int status)
+key_server_connect_callback(keysAsyncContext *keys, int status)
 {
     if (status == RESP_OK) {
 	if (pmDebugOptions.series)
@@ -154,31 +156,37 @@ keySlotsSetMetricRegistry(keySlots *slots, mmv_registry_t *registry)
     return -ENOMEM;
 }
 
-keySlots *
+keySlotsContext *
 keySlotsInit(dict *config, void *events)
 {
-    keySlots		*slots;
+    keySlotsContext		*context;
     sds			servers = NULL;
     sds			def_servers = NULL;
     sds			username = NULL;
     sds			password = NULL;
-    int			sts = 0;
-    struct timeval	connection_timeout = {5, 0}; // 5s
-    struct timeval	command_timeout = {60, 0}; // 1m
 
-    if ((slots = (keySlots *)calloc(1, sizeof(keySlots))) == NULL) {
-	pmNotifyErr(LOG_ERR, "%s: failed to allocate keySlots\n",
+    if ((context = (keySlotsContext *)calloc(1, sizeof(keySlotsContext))) == NULL) {
+	pmNotifyErr(LOG_ERR, "%s: failed to allocate keySlotsContext\n",
 			"keySlotsInit");
 	return NULL;
     }
 
-    slots->state = SLOTS_DISCONNECTED;
-    slots->events = events;
-    slots->keymap = dictCreate(&sdsKeyDictCallBacks, "keymap");
-    if (slots->keymap == NULL) {
+    context->slots.state = SLOTS_DISCONNECTED;
+    context->slots.events = events;
+    context->slots.keymap = malloc(sizeof(keyMap));
+    if (context->slots.keymap == NULL) {
 	pmNotifyErr(LOG_ERR, "%s: failed to allocate keymap\n",
-			"keySlotsInit");
-	free(slots);
+		"keySlotsInit");
+	free(context);
+	return NULL;
+    }
+    context->slots.keymap->dict = dictCreate(&sdsKeyDictCallBacks);
+    context->slots.keymap->privdata = NULL;
+    if (context->slots.keymap->dict == NULL) {
+	pmNotifyErr(LOG_ERR, "%s: failed to allocate keymap dict\n",
+		"keySlotsInit");
+	free(context->slots.keymap);
+	free(context);
 	return NULL;
     }
 
@@ -201,107 +209,53 @@ keySlotsInit(dict *config, void *events)
     if (password == NULL)
 	password = pmIniFileLookup(config, "pmseries", "auth.password");
 
-    if ((slots->acc = keyClusterAsyncContextInit()) == NULL) {
-	/* Coverity CID370635 */
-	pmNotifyErr(LOG_ERR, "%s: %s failed\n",
-			"keySlotsInit", "keyClusterAsyncContextInit");
-	sdsfree(def_servers);
-	return slots;
-    }
-
-    if (slots->acc->err) {
-        pmNotifyErr(LOG_ERR, "%s: %s\n", "keySlotsInit", slots->acc->errstr);
-	sdsfree(def_servers);
-	return slots;
-    }
-
-    sts = keyClusterSetOptionAddNodes(slots->acc->cc, servers);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to add key server nodes: %s\n",
-			"keySlotsInit", slots->acc->cc->errstr);
-	sdsfree(def_servers);
-	return slots;
-    }
-    sdsfree(def_servers); /* Coverity CID370634 */
-
     /*
+     * Add options to the valkeyClusterOptions struct
      * the ini parser already removes spaces at the beginning and end of the
      * configuration values, so checking for empty strings using sdslen() is
-     * fine
-     */
-    if (username != NULL && sdslen(username) > 0) {
-	sts = keyClusterSetOptionUsername(slots->acc->cc, username);
-	if (sts != RESP_OK) {
-	    pmNotifyErr(LOG_ERR, "%s: failed to set key server username: %s\n",
-		"keyClusterSetOptionUsername", slots->acc->cc->errstr);
-	    return slots;
-	}
-    }
-
-    /*
+     * fine.
      * see note above re empty configuration values having only a password
      * set and no username is a valid key server configuration, details:
      * https://valkey.io/commands/auth
-     */
-    if (password != NULL && sdslen(password) > 0) {
-	sts = keyClusterSetOptionPassword(slots->acc->cc, password);
-	if (sts != RESP_OK) {
-	    pmNotifyErr(LOG_ERR, "%s: failed to set key server password: %s\n",
-		"keyClusterSetOptionPassword", slots->acc->cc->errstr);
-	    return slots;
-	}
-    }
+    */
 
-    sts = keyClusterSetOptionConnectTimeout(slots->acc->cc, connection_timeout);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to set connect timeout: %s\n",
-			"keySlotsInit", slots->acc->errstr);
-	return slots;
-    }
+    /* Initialize timeout values in the context structure */
+    context->connection_timeout.tv_sec = 5;
+    context->connection_timeout.tv_usec = 0;
+    context->command_timeout.tv_sec = 60;
+    context->command_timeout.tv_usec = 0;
 
-    sts = keyClusterSetOptionTimeout(slots->acc->cc, command_timeout);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to set command timeout: %s\n",
-			"keySlotsInit", slots->acc->cc->errstr);
-	return slots;
-    }
+    context->opts.initial_nodes = servers;                    // sds string
+    context->opts.username = (username && sdslen(username)) ? username : NULL;
+    context->opts.password = (password && sdslen(password)) ? password : NULL;
+    context->opts.connect_timeout = &context->connection_timeout;      // struct timeval*
+    context->opts.command_timeout = &context->command_timeout;      // struct timeval*
+    context->opts.async_connect_callback = key_server_connect_callback;
+    context->opts.async_disconnect_callback = key_server_disconnect_callback;
+#if defined(HAVE_LIBUV)
+    context->opts.attach_fn = keyLibuvAttachAdapter;
+    context->opts.attach_data = (uv_loop_t *)events;
+#endif
 
-    sts = keyClusterLibuvAttach(slots->acc, slots->events);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to attach to event loop: %s\n",
-			"keySlotsInit", slots->acc->errstr);
-	return slots;
-    }
+    /* Note: actual connection happens in keySlotsReconnect(), not here.
+     * This avoids creating a context that gets immediately freed and recreated. */
+    context->slots.acc = NULL;
+    
+    sdsfree(def_servers); /* Coverity CID370634 */
 
-    sts = keyClusterAsyncSetConnectCallback(slots->acc, key_server_connect_callback);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to set connect callback: %s\n",
-			"keySlotsInit", slots->acc->errstr);
-	return slots;
-    }
-
-    sts = keyClusterAsyncSetDisconnectCallback(slots->acc, key_server_disconnect_callback);
-    if (sts != RESP_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to set disconnect callback: %s\n",
-			"keySlotsInit", slots->acc->errstr);
-	return slots;
-    }
-
-    return slots;
+    return context;
 }
 
 /**
- * despite the name, this function also handles the initial
- * connection to the key server
+ * Handles reconnection logic and schema loading after initial
+ * connection (which happens in keySlotsInit with libvalkey)
  */
 void
 keySlotsReconnect(keySlots *slots, keySlotsFlags flags,
 		keysInfoCallBack info, keysDoneCallBack done,
-		void *userdata, void *events, void *arg)
+		void *userdata, void *events, void *arg, valkeyClusterOptions *opts)
 {
-    dictIterator	*iterator;
     dictEntry		*entry;
-    int			sts = 0;
     static int		log_connection_errors = 1;
 
     if (slots == NULL)
@@ -310,28 +264,69 @@ keySlotsReconnect(keySlots *slots, keySlotsFlags flags,
     slots->state = SLOTS_CONNECTING;
     slots->conn_seq++;
 
-    /* reset key server context in case of reconnect */
-    if (slots->acc->err) {
-	/* reset possible 'Connection refused' error before reconnecting */
-	slots->acc->err = 0;
-	memset(slots->acc->errstr, '\0', strlen(slots->acc->errstr));
+    /* Free old async context if this is a reconnect */
+    if (slots->acc != NULL) {
+	/* reset key server context in case of reconnect */
+	if (slots->acc->err) {
+	    /* reset possible 'Connection refused' error before reconnecting */
+	    slots->acc->err = 0;
+	    memset(slots->acc->errstr, '\0', strlen(slots->acc->errstr));
+	}
+	keyClusterAsyncDisconnect(slots->acc);
+
+	/* reset keySlots in case of reconnect */
+	slots->cluster = 0;
+	slots->search = 0;
+	/* Clear the keymap by iterating and deleting all entries.
+	 * Note: Must free the values (malloc'd int64_t* pointers) before
+	 * deleting entries since sdsKeyDictCallBacks has no valDestructor. */
+	{
+	    dictIterator iter;
+	    dictEntry *e;
+	    dictInitIterator(&iter, slots->keymap->dict);
+	    while ((e = dictNext(&iter)) != NULL) {
+		void *val = dictGetVal(e);
+		if (val != NULL) {
+		    free(val);  /* Free the malloc'd int64_t* position pointer */
+		}
+		dictDelete(slots->keymap->dict, dictGetKey(e));
+	    }
+	}
+
+	/* Free old async context completely and create a new one.
+	 * libvalkey doesn't support reconnecting an existing async context,
+	 * so we need to free the old one and create a new one. */
+	keyClusterAsyncFree(slots->acc);
+	slots->acc = NULL;
     }
-    keyClusterAsyncDisconnect(slots->acc);
-
-    /* reset keySlots in case of reconnect */
-    slots->cluster = 0;
-    slots->search = 0;
-    dictEmpty(slots->keymap, NULL);
-
-    sts = keyClusterConnect2(slots->acc->cc);
-    if (sts == RESP_OK) {
+    
+    /* Enable blocking initial slot map update for reconnect so the context
+     * is immediately ready to accept commands when the function returns. */
+    int saved_options = opts->options;
+    opts->options |= KEYOPT_BLOCKING_INITIAL_UPDATE;
+    
+    /* Create new async connection using the same options */
+    slots->acc = keyClusterAsyncContextInit(opts);
+    
+    /* Restore original options */
+    opts->options = saved_options;
+    
+    if (slots->acc == NULL) {
+	pmNotifyErr(LOG_ERR, "%s: failed to create new async context\n",
+		    "keySlotsReconnect");
+	slots->state = SLOTS_DISCONNECTED;
+	return;
+    }
+    if (slots->acc->err == 0) {
 	slots->cluster = 1;
     }
-    else if (slots->acc->cc->err &&
-		strcmp(slots->acc->cc->errstr, RESP_ENOCLUSTER) == 0) {
+    else if (slots->acc->err &&
+		strcmp(slots->acc->errstr, RESP_ENOCLUSTER) == 0) {
 	/* key server instance has cluster support disabled */
-	slots->acc->cc->err = 0;
-	memset(slots->acc->cc->errstr, '\0', strlen(slots->acc->cc->errstr));
+	/* Clear error (following valkeyClusterAsyncClearError pattern) */
+	slots->acc->cc.err = 0;
+	slots->acc->cc.errstr[0] = '\0';
+	slots->acc->err = slots->acc->cc.err;  /* sync acc->err from cc->err */
 	slots->cluster = 0;
 
 	/*
@@ -339,21 +334,38 @@ keySlotsReconnect(keySlots *slots, keySlotsFlags flags,
 	 * is configured, but cluster mode is disabled
 	 * otherwise all other nodes silently don't get any data
 	 */
-	iterator = dictGetSafeIterator(slots->acc->cc->nodes);
-	entry = dictNext(iterator);
-	if (entry && dictNext(iterator)) {
-	    dictReleaseIterator(iterator);
-	    pmNotifyErr(LOG_ERR, "%s: more than one node is configured, "
-			"but cluster mode is disabled", "keySlotsReconnect");
-	    slots->state = SLOTS_ERR_FATAL;
-	    return;
+	{
+	    dictIterator iter;
+	    dictInitIterator(&iter, slots->acc->cc.nodes);
+	    entry = dictNext(&iter);
+	    if (entry && dictNext(&iter)) {
+		pmNotifyErr(LOG_ERR, "%s: more than one node is configured, "
+			    "but cluster mode is disabled", "keySlotsReconnect");
+		slots->state = SLOTS_ERR_FATAL;
+		return;
+	    }
 	}
-	dictReleaseIterator(iterator);
     }
     else {
 	if (log_connection_errors || pmDebugOptions.desperate) {
-	    pmNotifyErr(LOG_INFO, "Cannot connect to key server: %s\n",
-			slots->acc->cc->errstr);
+	    /* acc->errstr should point to acc->cc.errstr, but be defensive */
+	    const char *errstr = NULL;
+	    if (slots->acc->errstr != NULL && slots->acc->errstr[0] != '\0') {
+		errstr = slots->acc->errstr;
+	    } else if (slots->acc->cc.errstr[0] != '\0') {
+		errstr = slots->acc->cc.errstr;
+	    } else {
+		/* Check if we can get more info from errno or error code */
+		switch (slots->acc->err) {
+		    case RESP_ERR_IO:
+			errstr = "Connection refused";
+			break;
+		    default:
+			errstr = "Unknown error";
+			break;
+		}
+	    }
+	    pmNotifyErr(LOG_INFO, "Cannot connect to key server: %s\n", errstr);
 	    log_connection_errors = 0;
 	}
 	slots->state = SLOTS_DISCONNECTED;
@@ -370,12 +382,12 @@ keySlotsReconnect(keySlots *slots, keySlotsFlags flags,
  * compatibility, the actual connection to the key server happens in
  * keySlotsReconnect()
  */
-keySlots *
+keySlotsContext *
 keySlotsConnect(dict *config, keySlotsFlags flags,
 		keysInfoCallBack info, keysDoneCallBack done,
 		void *userdata, void *events, void *arg)
 {
-    keySlots			*slots;
+    keySlotsContext		*context;
     sds				enabled, msg;
 
     if (!(enabled = pmIniFileLookup(config, "resp", "enabled")))
@@ -383,8 +395,8 @@ keySlotsConnect(dict *config, keySlotsFlags flags,
     if (enabled && strcmp(enabled, "false") == 0)
 	return NULL;
 
-    slots = keySlotsInit(config, events);
-    if (slots == NULL) {
+    context = keySlotsInit(config, events);
+    if (context == NULL) {
 	msg = NULL;
 	infofmt(msg, "Failed to allocate memory for key server slots");
 	info(PMLOG_ERROR, msg, arg);
@@ -392,8 +404,8 @@ keySlotsConnect(dict *config, keySlotsFlags flags,
 	return NULL;
     }
 
-    keySlotsReconnect(slots, flags, info, done, userdata, events, arg);
-    return slots;
+    keySlotsReconnect(&context->slots, flags, info, done, userdata, events, arg, &context->opts);
+    return context;
 }
 
 void
@@ -401,11 +413,64 @@ keySlotsFree(keySlots *slots)
 {
     keyClusterAsyncDisconnect(slots->acc);
     keyClusterAsyncFree(slots->acc);
-    dictRelease(slots->keymap);
+    if (slots->keymap) {
+	if (slots->keymap->privdata)
+	    sdsfree((sds)slots->keymap->privdata);
+	/* Free all values (malloc'd int64_t* pointers) before releasing dict.
+	 * sdsKeyDictCallBacks has no valDestructor, so we must do this manually. */
+	if (slots->keymap->dict) {
+	    dictIterator iter;
+	    dictEntry *e;
+	    dictInitIterator(&iter, slots->keymap->dict);
+	    while ((e = dictNext(&iter)) != NULL) {
+		void *val = dictGetVal(e);
+		if (val != NULL) {
+		    free(val);
+		}
+	    }
+	    dictRelease(slots->keymap->dict);
+	}
+	free(slots->keymap);
+    }
     memset(slots, 0, sizeof(*slots));
     free(slots);
 }
 
+void
+keySlotsContextFree(keySlotsContext *context)
+{
+    if (context == NULL)
+	return;
+
+    /* Set state to disconnected BEFORE invoking callbacks to prevent
+     * callbacks from issuing new requests during shutdown */
+    context->slots.state = SLOTS_DISCONNECTED;
+
+    keyClusterAsyncDisconnect(context->slots.acc);
+    keyClusterAsyncFree(context->slots.acc);
+    if (context->slots.keymap) {
+	if (context->slots.keymap->privdata)
+	    sdsfree((sds)context->slots.keymap->privdata);
+	/* Free all values (malloc'd int64_t* pointers) before releasing dict.
+	 * sdsKeyDictCallBacks has no valDestructor, so we must do this manually. */
+	if (context->slots.keymap->dict) {
+	    dictIterator iter;
+	    dictEntry *e;
+	    dictInitIterator(&iter, context->slots.keymap->dict);
+	    while ((e = dictNext(&iter)) != NULL) {
+		void *val = dictGetVal(e);
+		if (val != NULL) {
+		    free(val);
+		}
+	    }
+	    dictRelease(context->slots.keymap->dict);
+	}
+	free(context->slots.keymap);
+    }
+    memset(context, 0, sizeof(*context));
+    free(context);
+}
+
 static inline uint64_t
 gettimeusec(void)
 {
@@ -567,7 +632,8 @@ keySlotsRequest(keySlots *slots, const sds cmd,
 		    keySlotsReplyCallback, srd, cmd, size)) != RESP_OK) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
 	pmNotifyErr(LOG_ERR, "%s: %s (%s)\n", "keySlotsRequest",
-			slots->acc->errstr, cmd);
+		slots->acc->errstr, cmd);
+	keySlotsReplyDataFree(srd);
 	return -ENOMEM;
     }
 
@@ -583,9 +649,8 @@ int
 keySlotsRequestFirstNode(keySlots *slots, const sds cmd,
 		keyClusterCallbackFn *callback, void *arg)
 {
-    dictIterator	*iterator;
     dictEntry		*entry;
-    cluster_node	*node;
+    valkeyClusterNode	*node;
     keySlotsReplyData	*srd;
     uint64_t		size;
     int			sts;
@@ -597,9 +662,11 @@ keySlotsRequestFirstNode(keySlots *slots, const sds cmd,
     if (UNLIKELY(slots->state != SLOTS_CONNECTED && slots->state != SLOTS_READY))
 	return -ENOTCONN;
 
-    iterator = dictGetSafeIterator(slots->acc->cc->nodes);
-    entry = dictNext(iterator);
-    dictReleaseIterator(iterator);
+    {
+	dictIterator iter;
+	dictInitIterator(&iter, slots->acc->cc.nodes);
+	entry = dictNext(&iter);
+    }
     if (!entry) {
 	pmNotifyErr(LOG_ERR, "%s: No key server node configured.",
 			"keySlotsRequestFirstNode");
@@ -619,11 +686,12 @@ keySlotsRequestFirstNode(keySlots *slots, const sds cmd,
 	return -ENOMEM;
     }
     sts = keyClusterAsyncFormattedCommandToNode(slots->acc, node,
-			keySlotsReplyCallback, srd, cmd, size);
+		keySlotsReplyCallback, srd, cmd, size);
     if (sts != RESP_OK) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
 	pmNotifyErr(LOG_ERR, "%s: %s (%s)\n",
-			"keySlotsRequestFirstNode", slots->acc->errstr, cmd);
+		"keySlotsRequestFirstNode", slots->acc->errstr, cmd);
+	keySlotsReplyDataFree(srd);
 	return -ENOMEM;
     }
 
@@ -681,8 +749,8 @@ keySlotsProxyConnect(keySlots *slots, keysInfoCallBack info,
 	    reply->type == RESP_REPLY_MAP ||
 	    reply->type == RESP_REPLY_SET)
 	    cmd = sdsnew(reply->element[0]->str);
-	if (cmd && (entry = dictFind(slots->keymap, cmd)) != NULL) {
-	    position = dictGetSignedIntegerVal(entry);
+	if (cmd && (entry = dictFind(slots->keymap->dict, cmd)) != NULL) {
+	    position = *(int64_t *)dictGetVal(entry);
 	    if (position > 0 && position < reply->elements)
 		hasKey = 1;
 	}
@@ -735,8 +803,8 @@ reportReplyError(keysInfoCallBack info, void *userdata,
 	msg = sdscatfmt(msg, "\nRESP reply error: %s", reply->str);
     else if (acc->err)
 	msg = sdscatfmt(msg, "\nRESP acc error: %s", acc->errstr);
-    else if (acc->cc->err)
-	msg = sdscatfmt(msg, "\nRESP cc error: %s", acc->cc->errstr);
+    else if (acc->cc.err)
+	msg = sdscatfmt(msg, "\nRESP cc error: %s", acc->cc.errstr);
     info(PMLOG_RESPONSE, msg, userdata);
     sdsfree(msg);
 }
