@@ -6,6 +6,10 @@ struct gpupidstat {
 	struct gpu	gpu;
 };
 
+void	setup_gpuphotoproc(void);
+
+unsigned long	gpuphotoproc(struct gpupidstat **, unsigned int *);
+
 void	gpumergeproc(struct tstat      *, int,
                      struct tstat      *, int,
                      struct gpupidstat *, int);
