@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #
 # Copyright (c) 2012 Nathan Scott.  All Rights Reserved.
 #
@@ -13,12 +13,37 @@
 # for more details.
 #
 
-# . $PCP_DIR/etc/pcp.sh
+# --- start common preamble
+if [ -z "$PCP_SH_DONE" ]
+then
+    if [ -n "$PCP_CONF" ]
+    then
+	__CONF="$PCP_CONF"
+    elif [ -n "$PCP_DIR" ]
+    then
+	__CONF="$PCP_DIR/etc/pcp.conf"
+    else
+	__CONF=/etc/pcp.conf
+    fi
+    if [ ! -f "$__CONF" ]
+    then
+	echo "pcp.env: Fatal Error: \"$__CONF\" not found" >&2
+	exit 1
+    fi
+    eval `sed -e 's/"//g' $__CONF \
+    | awk -F= '
+/^PCP_/ && NF == 2 {
+	exports=exports" "$1
+	printf "%s=${%s:-\"%s\"}\n", $1, $1, $2
+} END {
+	print "export", exports
+}'`
+    PCP_ENV_DONE=y; export PCP_ENV_DONE
+fi
+. $PCP_SHARE_DIR/lib/bashproc.sh
+# --- end common preamble
 
-. $PCP_DIR/etc/pcp.env
-. ./bashproc.sh
-
-pcp_trace on $@
+pcp_trace on $0 $@
 
 tired()
 {
@@ -42,5 +67,4 @@ do
 done
 
 pcp_trace off
-
 exit 0
