@@ -1,7 +1,7 @@
 /*
  * Device Mapper PMDA
  *
- * Copyright (c) 2015,2018-2019 Red Hat.
+ * Copyright (c) 2015,2018-2019, 2025 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -23,16 +23,22 @@
 #include "dmcache.h"
 #include "dmstats.h"
 #include "vdo.h"
+#include "dmcrypt.h"
+#include "dmmultipath.h"
 
 static int		_isDSO = 1; /* for local contexts */
 
 enum {
-    CLUSTER_CACHE = 0,		/* DM-Cache Caches */
-    CLUSTER_POOL = 1,		/* DM-Thin Pools */
-    CLUSTER_VOL = 2,		/* DM-Thin Volumes */
-    CLUSTER_DM_COUNTER = 3,	/* Dmstats basic counter */
-    CLUSTER_DM_HISTOGRAM = 4,	/* Dmstats latency histogram */
-    CLUSTER_VDODEV = 5,		/* VDO per-device statistics */
+    CLUSTER_CACHE = 0,			/* DM-Cache Caches */
+    CLUSTER_POOL = 1,			/* DM-Thin Pools */
+    CLUSTER_VOL = 2,			/* DM-Thin Volumes */
+    CLUSTER_DM_COUNTER = 3,		/* Dmstats basic counter */
+    CLUSTER_DM_HISTOGRAM = 4,		/* Dmstats latency histogram */
+    CLUSTER_VDODEV = 5,			/* VDO per-device statistics */
+    CLUSTER_DM_CRYPT = 6, 		/* DM-Crypt Targets */
+    CLUSTER_DM_MULTIPATH_INFO = 7,	/* DM-Multipath Info */
+    CLUSTER_DM_MULTIPATH_PATH = 8,	/* DM-Multipath Path */
+    CLUSTER_DM_MULTIPATH_DEVICE = 9,	/* DM-Mulitpath Device */
     NUM_CLUSTERS
 };
 
@@ -1036,6 +1042,133 @@ static pmdaMetric metrictable[] = {
         PMDA_PMID(CLUSTER_VDODEV, VDODEV_SAVINGS_PERCENTAGE),
         PM_TYPE_FLOAT, DM_VDODEV_INDOM, PM_SEM_INSTANT,
         PMDA_PMUNITS(0,0,0,0,0,0) }, },
+
+    /* DM Crypt stats */
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_ACTIVE),
+        PM_TYPE_U32, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_TYPE),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_CIPHER),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_KEYSIZE),
+        PM_TYPE_U32, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_KEY_LOCATION),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_DEVICE),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_SECTOR_SIZE),
+        PM_TYPE_U32, DM_CRYPT_INDOM, PM_SEM_DISCRETE,
+        PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_OFFSET),
+        PM_TYPE_U64, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_OFFSET_BYTES),
+        PM_TYPE_U64, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_SIZE),
+        PM_TYPE_U64, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_SIZE_BYTES),
+        PM_TYPE_U64, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_MODE),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_CRYPT, CRYPT_FLAGS),
+        PM_TYPE_STRING, DM_CRYPT_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    /* DM Multipath stats */
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_NAME),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_WWID),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_DEVICE),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_VENDOR),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_PRODUCT_NAME),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_SIZE),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_FEATURES),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_HARDWARE_HANDLER),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_INFO, MULTIPATH_PERMISSIONS),
+        PM_TYPE_STRING, DM_MULTI_INFO_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_PATH, MULTIPATH_PATH_SELECTOR_ALGORITHM),
+        PM_TYPE_STRING, DM_MULTI_PATH_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_PATH, MULTIPATH_PATH_PRIORITY),
+        PM_TYPE_U64, DM_MULTI_PATH_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_PATH, MULTIPATH_PATH_STATUS),
+        PM_TYPE_STRING, DM_MULTI_PATH_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_BUS_ID),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_DEV_NAME),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_DEV_ID),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_DEVICE_STATUS),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_PATH_STATUS),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
+    { .m_desc = {
+        PMDA_PMID(CLUSTER_DM_MULTIPATH_DEVICE, MULTIPATH_DEVICE_KERNEL_STATUS),
+        PM_TYPE_STRING, DM_MULTI_DEV_INDOM, PM_SEM_INSTANT,
+        PMDA_PMUNITS(0,0,0,0,0,0) }, },
 };
 
 static pmdaIndom indomtable[] = {
@@ -1045,6 +1178,10 @@ static pmdaIndom indomtable[] = {
     { .it_indom = DM_STATS_INDOM },
     { .it_indom = DM_HISTOGRAM_INDOM },
     { .it_indom = DM_VDODEV_INDOM },
+    { .it_indom = DM_CRYPT_INDOM },
+    { .it_indom = DM_MULTI_INFO_INDOM },
+    { .it_indom = DM_MULTI_PATH_INDOM },
+    { .it_indom = DM_MULTI_DEV_INDOM },
 };
 
 pmInDom
@@ -1065,6 +1202,8 @@ dm_instance(pmInDom indom, int inst, char *name, pmInResult **result, pmdaExt *p
 	dm_cache_instance_refresh();
 	dm_thin_pool_instance_refresh();
 	dm_thin_vol_instance_refresh();
+	dm_crypt_instance_refresh();
+	dm_multipath_instance_refresh();
     }
 
     (void)pm_dm_stats_instance_refresh();
@@ -1180,6 +1319,31 @@ dm_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 	dm_vdodev_instance_refresh();
     }
 
+    if (need_refresh[CLUSTER_DM_CRYPT] && privilege) {
+        struct crypt_stats *crypt;
+
+        if ((sts = dm_crypt_instance_refresh()) < 0)
+	    return sts;
+
+        indom = dm_indom(DM_CRYPT_INDOM);
+
+        for (pmdaCacheOp(indom, PMDA_CACHE_WALK_REWIND);;) {
+	    if ((sts = pmdaCacheOp(indom, PMDA_CACHE_WALK_NEXT)) < 0)
+	        break;
+	    if (!pmdaCacheLookup(indom, sts, &name, (void **)&crypt) || !crypt)
+	        continue;
+            if (need_refresh[CLUSTER_DM_CRYPT])
+                dm_refresh_crypt(name, crypt);
+        }
+    }
+    
+    if ((need_refresh[CLUSTER_DM_MULTIPATH_INFO] ||
+         need_refresh[CLUSTER_DM_MULTIPATH_PATH] ||
+         need_refresh[CLUSTER_DM_MULTIPATH_DEVICE]) && privilege) {
+         if ((sts = dm_multipath_instance_refresh()) < 0)
+             return sts;
+    }
+
     return 0;
 }
 
@@ -1211,6 +1375,10 @@ dm_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     struct pool_stats *pool;
     struct vol_stats *vol;
     struct pm_wrap *pw;
+    struct crypt_stats *crypt;
+    struct multipath_info *info;
+    struct multipath_path *path;
+    struct multipath_device *dev;
     char *device;
     int sts;
 
@@ -1251,6 +1419,30 @@ dm_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	        return sts;
 	    return dm_vdodev_fetch(mdesc, device, atom);
 
+        case CLUSTER_DM_CRYPT:
+	    sts = pmdaCacheLookup(dm_indom(DM_CRYPT_INDOM), inst, NULL, (void **)&crypt);
+	    if (sts < 0)
+	        return sts;
+	    return dm_crypt_fetch(item, crypt, atom);
+
+        case CLUSTER_DM_MULTIPATH_INFO:
+	    sts = pmdaCacheLookup(dm_indom(DM_MULTI_INFO_INDOM), inst, NULL, (void **)&info);
+	    if (sts < 0)
+	        return sts;
+	    return dm_multipath_info_fetch(item, info, atom);
+
+        case CLUSTER_DM_MULTIPATH_PATH:
+	    sts = pmdaCacheLookup(dm_indom(DM_MULTI_PATH_INDOM), inst, NULL, (void **)&path);
+	    if (sts < 0)
+	        return sts;
+	    return dm_multipath_path_fetch(item, path, atom);
+
+        case CLUSTER_DM_MULTIPATH_DEVICE:
+	    sts = pmdaCacheLookup(dm_indom(DM_MULTI_DEV_INDOM), inst, NULL, (void **)&dev);
+	    if (sts < 0)
+	        return sts;
+	    return dm_multipath_device_fetch(item, dev, atom);
+
         default: /* unknown cluster */
 	    return PM_ERR_PMID;
     }
@@ -1283,6 +1475,8 @@ dm_init(pmdaInterface *dp)
     dm_cache_setup();
     dm_thin_setup();
     dm_vdo_setup();
+    dm_crypt_setup();
+    dm_multipath_setup();
 
     int	nindoms = sizeof(indomtable)/sizeof(indomtable[0]);
     int	nmetrics = sizeof(metrictable)/sizeof(metrictable[0]);
