@@ -41,6 +41,7 @@ our $dffilter = '(objectclass=*)';
 our $dattrs = ['*', '+'];
 # How often it will check
 our $query_interval = 2; # seconds
+our $query_timeout = 1; # seconds
 # Metrics defaults
 our $mpm_type = PM_TYPE_U32;
 our $mpm_indom = PM_INDOM_NULL;
@@ -194,10 +195,10 @@ use vars qw( $ldap $pmda %metrics );
 
 sub ds389_connection_setup {
   if (!defined($ldap)) {
-    $pmda->log("binding to $server");
-    $ldap = Net::LDAP->new($server,version => $ldapver);
+    if (!pmda_install()) { $pmda->log("binding to $server"); }
+    $ldap = Net::LDAP->new($server, version => $ldapver, timeout => $query_timeout);
     if (!defined($ldap)) {
-      $pmda->log("bind failed, server down?");
+      if (!pmda_install()) { $pmda->log("bind failed, server down?"); }
       return;
     }
     my $mesg = $ldap->bind($binddn, password => $bindpw);
@@ -262,9 +263,9 @@ sub ds389_process_entry {
       if ($attr eq 'nsds5replicaLastUpdateStatus') {
         if ($value =~ /No replication sessions started since server startup/i) {
           $value = 30
-        } elif ($value =~ /agreement disabled/i) {
+        } elsif ($value =~ /agreement disabled/i) {
           $value = 31
-        } elif ($value =~ /Problem connecting to the replica/i) {
+        } elsif ($value =~ /Problem connecting to the replica/i) {
           $value = 20
         } else {
           $value = (split /\)/, (split /Error \(/, $value)[1])[0];
@@ -326,7 +327,7 @@ sub retrieve_ldap {
   
   if ((strftime("%s", localtime()) - $ts) > $query_interval) {
     $ts = strftime("%s", localtime());
-    $mesg = $ldap->search(scope => $scope, base => $base, filter => $filter, attrs => $lattrs);
+    $mesg = $ldap->search(scope => $scope, base => $base, filter => $filter, attrs => $lattrs, timelimit => $query_timeout);
     if ($mesg->code) {
       $pmda->log("search(scope: \"$scope\", base: \"$base\", filter: \"$filter\", attrs: \"". join(' ', @$lattrs) ."\") failed: " . $mesg->error);
       undef $ldap;
@@ -366,6 +367,7 @@ sub ds389_fetch_callback {
 sub ds389_simple_search {
   my ($scope, $base, $filter, $attrs) = @_;
 
+  if (!defined($ldap)) { return; }
   my $mesg = $ldap->search(scope => $scope, base => $base, filter => $filter, attrs => $attrs);
   if ($mesg->code) {
     $pmda->log("search(scope: \"$scope\", base: \"$base\", filter: \"$filter\", attrs: \"". join(' ', @$attrs) ."\") failed: " . $mesg->error);
@@ -393,19 +395,11 @@ foreach my $attr (keys %dataclusters) {
   }
 };
 
-$ldap = Net::LDAP->new($server,version => $ldapver);
-if (!defined($ldap)) {
-  $pmda->log("bind failed, server down?");
-}
+ds389_connection_setup();
 
-my $mesg = $ldap->bind($binddn, password => $bindpw);
-if ($mesg->code) {
-  $pmda->log("bind failed: " . $mesg->error);
-}
+my $mesg = ds389_simple_search('sub','cn=config','objectclass=*',['nsslapd-defaultnamingcontext','nsslapd-backend']);
 
-$mesg = ds389_simple_search('sub','cn=config','objectclass=*',['nsslapd-defaultnamingcontext','nsslapd-backend']);
-
-my $max = $mesg->count;
+my $max = defined($mesg) ? $mesg->count : 0;
 for ( my $i = 0 ; $i < $max ; $i++ ) {
   my $entry = $mesg->entry ( $i );
   foreach my $attr ($entry->attributes) {
