@@ -1,6 +1,6 @@
 #
-# Copyright (c) 2012 Chandana De Silva.
-# Copyright (c) 2012 Red Hat.
+# Copyright (c) 2012,2017 Red Hat.
+# Copyright (c) 2012-2013 Chandana De Silva.
 # Copyright (c) 2008 Aconex.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
@@ -22,6 +22,7 @@ use DBI;
 my $database = 'DBI:mysql:mysql';
 my $username = 'dbmonitor';
 my $password = 'dbmonitor';
+my $account = 'pcp';
 
 # Configuration files for overriding the above settings
 for my $file (	'/etc/pcpdbi.conf',	# system defaults (lowest priority)
@@ -35,22 +36,40 @@ use vars qw( $dbh $sth_variables $sth_status $sth_processes $sth_slave_status );
 my $process_indom = 0;
 my @process_instances;
 
+# translate yes/no true/false on/off to 1/0
+sub mysql_txt2num {
+    my ($value) = lc($_[0]);
+
+    if (!defined($value)) {
+        return (PM_ERR_AGAIN, 0);
+    }
+    elsif ($value eq "yes" || $value eq "true" || $value eq "on") {
+        return 1;
+    }
+    elsif ($value eq "no" || $value eq "false" || $value eq "off") {
+        return 0;
+    }
+    else {
+        return -1;
+    }
+}
+
 sub mysql_connection_setup
 {
     # $pmda->log("mysql_connection_setup\n");
 
     if (!defined($dbh)) {
-	$dbh = DBI->connect($database, $username, $password);
-	if (defined($dbh)) {
-	    # set the db handle to undef in case of any failure
-	    # this will force a database reconnect
-	    $dbh->{HandleError} = sub { $dbh = undef; };
-	    $pmda->log("MySQL connection established\n");
-	    $sth_variables = $dbh->prepare('show variables');
-	    $sth_status = $dbh->prepare('show status');
-	    $sth_processes = $dbh->prepare('show processlist');
-	    $sth_slave_status = $dbh->prepare('show slave status');
-	}
+        $dbh = DBI->connect($database, $username, $password);
+        if (defined($dbh)) {
+            # set the db handle to undef in case of any failure
+            # this will force a database reconnect
+            $dbh->{HandleError} = sub { $dbh = undef; };
+            $pmda->log("MySQL connection established\n");
+            $sth_variables = $dbh->prepare('show variables');
+            $sth_status = $dbh->prepare('show global status');
+            $sth_processes = $dbh->prepare('show processlist');
+            $sth_slave_status = $dbh->prepare('show slave status');
+        }
     }
 }
 
@@ -60,11 +79,11 @@ sub mysql_variables_refresh
 
     %variables = ();	# clear any previous contents
     if (defined($dbh)) {
-	$sth_variables->execute();
-	my $result = $sth_variables->fetchall_arrayref();
-	for my $i (0 .. $#{$result}) {
-	    $variables{$result->[$i][0]} = $result->[$i][1];
-	}
+        $sth_variables->execute();
+        my $result = $sth_variables->fetchall_arrayref();
+        for my $i (0 .. $#{$result}) {
+            $variables{$result->[$i][0]} = $result->[$i][1];
+        }
     }
 }
 
@@ -74,12 +93,20 @@ sub mysql_status_refresh
 
     %status = ();	# clear any previous contents
     if (defined($dbh)) {
-	$sth_status->execute();
-	my $result = $sth_status->fetchall_arrayref();
-	for my $i (0 .. $#{$result}) {
-	    my $key = lcfirst $result->[$i][0];
-	    $status{$key} = $result->[$i][1];
-	}
+        $sth_status->execute();
+        my $result = $sth_status->fetchall_arrayref();
+        my $txtnum;
+        my $txtnumvar;
+        for my $i (0 .. $#{$result}) {
+            my $key = lcfirst $result->[$i][0];
+            $status{$key} = $result->[$i][1];
+            # if this status value has a yes/no type value, get it translated
+            $txtnum = mysql_txt2num($result->[$i][1]);
+            if ($txtnum ge 0) {
+                $txtnumvar=$key . "_num";
+                $status{$txtnumvar} = $txtnum;
+            }
+        }
     }
 }
 
@@ -91,13 +118,13 @@ sub mysql_process_refresh
     @process_instances = ();	# refresh indom too
 
     if (defined($dbh)) {
-	$sth_processes->execute();
-	my $result = $sth_processes->fetchall_arrayref();
-	for my $i (0 .. $#{$result}) {
-	    $process_instances[($i*2)] = $i;
-	    $process_instances[($i*2)+1] = "$result->[$i][0]";
-	    $processes[$i] = $result->[$i];
-	}
+        $sth_processes->execute();
+        my $result = $sth_processes->fetchall_arrayref();
+        for my $i (0 .. $#{$result}) {
+            $process_instances[($i*2)] = $i;
+            $process_instances[($i*2)+1] = "$result->[$i][0]";
+            $processes[$i] = $result->[$i];
+        }
     }
 
     $pmda->replace_indom($process_indom, \@process_instances);
@@ -109,11 +136,19 @@ sub mysql_slave_status_refresh
 
     %slave_status = ();	# clear any previous contents
     if (defined($dbh)) {
-	$sth_slave_status->execute();
-	my $result = $sth_slave_status->fetchrow_hashref();
-	while ( my ($key, $value) = each(%$result) ) {
-	    $slave_status{lc $key} = $value;
-	}
+        $sth_slave_status->execute();
+        my $result = $sth_slave_status->fetchrow_hashref();
+        my $txtnum;
+        my $txtnumvar;
+        while ( my ($key, $value) = each(%$result) ) {
+            $slave_status{lc $key} = $value;
+            # if this status value has a yes/no type value, get it translated
+            $txtnum = mysql_txt2num($value);
+            if ($txtnum ge 0) {
+                $txtnumvar=lc($key) . "_num";
+                $slave_status{$txtnumvar} = $txtnum;
+            }
+        }
     }
 }
 
@@ -147,26 +182,26 @@ sub mysql_fetch_callback
 	@procs = @$value;
 	if (!defined($procs[$item]) && $item == 6) { return ("?", 1); }
 	if (!defined($procs[$item])) { return (PM_ERR_APPVERSION, 0); }
-	return ($procs[$item], 1);
+    	return ($procs[$item], 1);
     }
     if ($inst != PM_IN_NULL)		{ return (PM_ERR_INST, 0); }
     if ($cluster == 0) {
-	$mysql_name =~ s/^mysql\.status\.//;
-	$value = $status{$mysql_name};
-	if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
-	return ($value, 1);
+        $mysql_name =~ s/^mysql\.status\.//;
+        $value = $status{$mysql_name};
+        if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
+        return ($value, 1);
     }
     elsif ($cluster == 1) {
-	$mysql_name =~ s/^mysql\.variables\.//;
-	$value = $variables{$mysql_name};
-	if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
-	return ($value, 1);
+        $mysql_name =~ s/^mysql\.variables\.//;
+        $value = $variables{$mysql_name};
+        if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
+        return ($value, 1);
     }
     elsif ($cluster == 3) {
-	$mysql_name =~ s/^mysql\.slave_status\.//;
-	$value = $slave_status{$mysql_name};
-	if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
-	return ($value, 1);
+        $mysql_name =~ s/^mysql\.slave_status\.//;
+        $value = $slave_status{$mysql_name};
+        if (!defined($value))	{ return (PM_ERR_APPVERSION, 0); }
+        return ($value, 1);
     }
     return (PM_ERR_PMID, 0);
 }
@@ -504,7 +539,7 @@ $pmda->add_metric(pmda_pmid(0,109), PM_TYPE_STRING, PM_INDOM_NULL,
 		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
 		  'mysql.status.compression', '', '');
 $pmda->add_metric(pmda_pmid(0,110), PM_TYPE_U32, PM_INDOM_NULL,
-		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
+		  PM_SEM_COUNTER, pmda_units(0,0,1,0,0,PM_COUNT_ONE),
 		  'mysql.status.connections', '', '');
 $pmda->add_metric(pmda_pmid(0,111), PM_TYPE_U32, PM_INDOM_NULL,
 		  PM_SEM_COUNTER, pmda_units(0,0,1,0,0,PM_COUNT_ONE),
@@ -927,11 +962,17 @@ $pmda->add_metric(pmda_pmid(0,250), PM_TYPE_U32, PM_INDOM_NULL,
 		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
 		  'mysql.status.threads_running', '', '');
 $pmda->add_metric(pmda_pmid(0,251), PM_TYPE_U32, PM_INDOM_NULL,
-		  PM_SEM_COUNTER, pmda_units(0,1,0,0,PM_TIME_SEC,0),
+		  PM_SEM_INSTANT, pmda_units(0,1,0,0,PM_TIME_SEC,0),
 		  'mysql.status.uptime', '', '');
 $pmda->add_metric(pmda_pmid(0,252), PM_TYPE_U32, PM_INDOM_NULL,
-		  PM_SEM_COUNTER, pmda_units(0,1,0,0,PM_TIME_SEC,0),
+		  PM_SEM_INSTANT, pmda_units(0,1,0,0,PM_TIME_SEC,0),
 		  'mysql.status.uptime_since_flush_status', '', '');
+$pmda->add_metric(pmda_pmid(0,253), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
+		  'mysql.status.slave_running_num', '', '');
+$pmda->add_metric(pmda_pmid(0,254), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
+		  'mysql.status.compression_num', '', '');
 
 $pmda->add_metric(pmda_pmid(1,0), PM_TYPE_U32, PM_INDOM_NULL,
 		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
@@ -1751,6 +1792,21 @@ $pmda->add_metric(pmda_pmid(3,30), PM_TYPE_U32, PM_INDOM_NULL,
 $pmda->add_metric(pmda_pmid(3,31), PM_TYPE_STRING, PM_INDOM_NULL,
 		  PM_SEM_DISCRETE, pmda_units(0,0,0,0,0,0),
 		  'mysql.slave_status.master_ssl_key', '', '');
+$pmda->add_metric(pmda_pmid(3,32), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
+		  'mysql.slave_status.slave_io_running_num', '', '');
+$pmda->add_metric(pmda_pmid(3,33), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,0,0,0,0,0),
+		  'mysql.slave_status.slave_sql_running_num', '', '');
+$pmda->add_metric(pmda_pmid(3,34), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_DISCRETE, pmda_units(0,0,0,0,0,0),
+		  'mysql.slave_status.master_ssl_allowed_num', '','');
+$pmda->add_metric(pmda_pmid(3,35), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,1,0,0,PM_TIME_SEC,0),
+		  'mysql.slave_status.sql_delay', '','');
+$pmda->add_metric(pmda_pmid(3,36), PM_TYPE_U32, PM_INDOM_NULL,
+		  PM_SEM_INSTANT, pmda_units(0,1,0,0,PM_TIME_SEC,0),
+		  'mysql.slave_status.sql_remaining_delay', '','');
 		  
 $pmda->add_indom($process_indom, \@process_instances,
 		 'Instance domain exporting each MySQL process', '');
@@ -1758,87 +1814,5 @@ $pmda->add_indom($process_indom, \@process_instances,
 $pmda->set_fetch_callback(\&mysql_fetch_callback);
 $pmda->set_fetch(\&mysql_connection_setup);
 $pmda->set_refresh(\&mysql_refresh);
-$pmda->set_user('pcp');
+$pmda->set_user($account);
 $pmda->run;
-
-=pod
-
-=head1 NAME
-
-pmdamysql - MySQL database PMDA
-
-=head1 DESCRIPTION
-
-B<pmdamysql> is a Performance Co-Pilot PMDA which extracts
-live performance data from a running MySQL database.
-
-=head1 INSTALLATION
-
-B<pmdamysql> uses a configuration file from (in this order):
-
-=over
-
-=item * /etc/pcpdbi.conf
-
-=item * $PCP_PMDAS_DIR/mysql/mysql.conf
-
-=back
-
-This file can contain overridden values (Perl code) for the settings
-listed at the start of pmdamysql.pl, namely:
-
-=over
-
-=item * database name (see DBI(3) for details)
-
-=item * database user name
-
-=item * database pass word
-
-=back
-
-Once this is setup, you can access the names and values for the
-mysql performance metrics by doing the following as root:
-
-	# cd $PCP_PMDAS_DIR/mysql
-	# ./Install
-
-If you want to undo the installation, do the following as root:
-
-	# cd $PCP_PMDAS_DIR/mysql
-	# ./Remove
-
-B<pmdamysql> is launched by pmcd(1) and should never be executed
-directly.  The Install and Remove scripts notify pmcd(1) when
-the agent is installed or removed.
-
-=head1 FILES
-
-=over
-
-=item /etc/pcpdbi.conf
-
-configuration file for all PCP database monitors
-
-=item $PCP_PMDAS_DIR/mysql/mysql.conf
-
-configuration file for B<pmdamysql>
-
-=item $PCP_PMDAS_DIR/mysql/Install
-
-installation script for the B<pmdamysql> agent
-
-=item $PCP_PMDAS_DIR/mysql/Remove
-
-undo installation script for the B<pmdamysql> agent
-
-=item $PCP_LOG_DIR/pmcd/mysql.log
-
-default log file for error messages from B<pmdamysql>
-
-=back
-
-=head1 SEE ALSO
-
-pmcd(1), pmdadbping.pl(1) and DBI(3).
-# vi: sw=4 ts=4 et
