@@ -31,7 +31,7 @@ static char	mypath[MAXPATHLEN];
  * ... real callback is fetch_callback()
  */
 static int
-solaris_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
+solaris_fetch(int numpmid, pmID pmidlist[], pmdaResult **resp, pmdaExt *pmda)
 {
     int		i;
 
@@ -127,6 +127,7 @@ usage(void)
 {
     fprintf(stderr, "Usage: %s [options]\n\n", pmGetProgname());
     fputs("Options:\n"
+	  "  -D debug    set debug options, see pmdbg(1)\n"
 	  "  -d domain    use domain (numeric) for metrics domain of PMDA\n"
 	  "  -l logfile   write log into logfile rather than using default log name\n"
 	  "  -N namespace verify consistency of internal metrics with the namespace\n", 
