@@ -51,15 +51,9 @@ init_pmda_metrics(struct agent_config* config) {
         (struct pmda_metrics_container*) malloc(sizeof(struct pmda_metrics_container));
     ALLOC_CHECK(container, "Unable to create PMDA metrics container.");
     pthread_mutex_init(&container->mutex, NULL);
-    struct pmda_metrics_dict_privdata* dict_data = 
-        (struct pmda_metrics_dict_privdata*) malloc(sizeof(struct pmda_metrics_dict_privdata));    
-    ALLOC_CHECK(dict_data, "Unable to create priv PMDA metrics container data.");
-    dict_data->config = config;
-    dict_data->container = container;
-    metrics* m = dictCreate(&metric_dict_callbacks, dict_data);
+    metrics* m = dictCreate(&metric_dict_callbacks);
     container->metrics = m;
     container->generation = 0;
-    container->metrics_privdata = dict_data;
     return container;
 }
 
@@ -232,11 +226,12 @@ write_metrics_to_file(struct agent_config* config, struct pmda_metrics_container
         VERBOSE_LOG(0, "Unable to open file for output.");
         return;
     }
-    dictIterator* iterator = dictGetSafeIterator(m);
+    dictIterator iterator;
+    dictInitIterator(&iterator, m);
     dictEntry* current;
     long int count = 0;
-    while ((current = dictNext(iterator)) != NULL) {
-        struct metric* item = (struct metric*)current->v.val;
+    while ((current = dictNext(&iterator)) != NULL) {
+        struct metric* item = (struct metric*)dictGetVal(current);
         switch (item->type) {
             case METRIC_TYPE_COUNTER:
                 print_counter_metric(config, f, item);
@@ -253,7 +248,6 @@ write_metrics_to_file(struct agent_config* config, struct pmda_metrics_container
         }
         count++;
     }
-    dictReleaseIterator(iterator);
     fprintf(f, "----------------\n");
     fprintf(f, "Total number of records: %lu \n", count);
     fclose(f);    
@@ -280,7 +274,7 @@ find_metric_by_name(struct pmda_metrics_container* container, char* key, struct
         return 0;
     }
     if (out != NULL) {
-        struct metric* item = (struct metric*)result->v.val;
+        struct metric* item = (struct metric*)dictGetVal(result);
         *out = item;
     }
     pthread_mutex_unlock(&container->mutex);
@@ -306,6 +300,7 @@ create_metric(struct agent_config* config, struct statsd_datagram* datagram, str
     (*out)->meta = create_metric_meta(datagram);
     (*out)->children = NULL;
     (*out)->committed = 0;
+    (*out)->config = config;
     int status = 0; 
     (*out)->value = NULL;
     // this metric doesn't have root value
