@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2020 Red Hat.
  * Copyright (c) 2019 Miroslav Foltýn.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -14,22 +15,22 @@
 #ifndef AGGREGATOR_METRIC_DICT_CALLBACKS_
 #define AGGREGATOR_METRIC_DICT_CALLBACKS_
 
-void
-metric_label_free_callback(void* privdata, void* val);
+extern void
+metric_label_free_callback(void* val);
 
-void
-metric_free_callback(void* privdata, void* val);
+extern void
+metric_free_callback(void* val);
 
-void
-str_hash_free_callback(void* privdata, void* key);
+extern void
+str_hash_free_callback(void* key);
 
-void*
-str_duplicate_callback(void* privdata, const void* key);
+extern void*
+str_duplicate_callback(const void* key);
 
-int
-str_compare_callback(void* privdata, const void* key1, const void* key2);
+extern int
+str_compare_callback(const void* key1, const void* key2);
 
-uint64_t
+extern uint64_t
 str_hash_callback(const void* key);
 
 #endif
