@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include "dstruct.h"
 
@@ -28,4 +28,5 @@ void showValue(FILE *, Expr *);
 void showAnnotatedValue(FILE *, Expr *);
 void showSatisfyingValue(FILE *, Expr *);
 void showTime(FILE *, RealTime);
+void showFullTime(FILE *, RealTime);
 size_t formatSatisfyingValue(char *, size_t, char **);
