@@ -14,6 +14,7 @@ extern int		meta_state;
 extern int		log_state;
 extern int		mark_count;
 extern int		result_count;
+extern int		repair;
 
 extern char *		goldenfname;
 extern int		goldenmagic;
@@ -23,4 +24,5 @@ extern int pass0(char *);
 extern int pass1(__pmContext *, char *);
 extern int pass2(__pmContext *, char *);
 extern int pass3(__pmContext *, char *, pmOptions *);
-
+extern int prompt(char *, char *);
+extern int try_truncate(char *, off_t, off_t);
