@@ -77,7 +77,7 @@ _cleanup()
     lockfile=`cat $tmp/lock 2>/dev/null`
     [ -n "$lockfile" ] && rm -f "$lockfile"
     rm -rf $tmp
-    $VERBOSE && echo >&2 "End [janitor]: `_datestamp` status=$status"
+    $VERBOSE && echo "End [janitor]: `_datestamp` status=$status"
 }
 
 trap "_cleanup; exit \$status" 0 1 2 3 15
@@ -230,7 +230,7 @@ else
     exec 3>&2 1>"$MYPROGLOG" 2>&1
 fi
 
-$VERBOSE && echo >&2 "Start [janitor]: `_datestamp`"
+$VERBOSE && echo "Start [janitor]: `_datestamp`"
 $VERY_VERBOSE && _pstree_all $$
 
 # if SaveLogs exists in the $PCP_LOG_DIR/pmlogger directory and is writeable
@@ -348,13 +348,40 @@ _callback_log_control()
 
     if [ -n "$pid" ]
     then
-	# found matching pmlogger ... cull this one from
-	$VERY_VERBOSE && echo "[$filename:$line] match PID $pid, nothing to be done"
+	# found matching pmlogger ... cull this one from $tmp/loggers
+	#
+	if $VERY_VERBOSE
+	then
+	    echo "[$filename:$line] match PID $pid, nothing to be done"
+	elif $VERBOSE
+	then
+	    echo "Pass 3: PID $pid matches control [$filename:$line], nothing to be done"
+	fi
 	sed <$tmp/loggers >$tmp/tmp -e "/^$pid	/d"
 	mv $tmp/tmp $tmp/loggers
     fi
 }
 
+# for $VERBOSE debugging ...
+# report ps(1) info for pids from $1 file (in $tmp/loggers format)
+#
+_debug_report()
+{
+    $PCP_PS_PROG $PCP_PS_ALL_FLAGS >$tmp/ps-all 2>&1
+    head -1 $tmp/ps-all
+    $PCP_AWK_PROG  <$1 '{ print $1 }' \
+    | while read pid
+    do
+	$PCP_AWK_PROG "\$2 == $pid { print }" <$tmp/ps-all >$tmp/debug
+	if [ -s $tmp/debug ]
+	then
+	    cat $tmp/debug
+	else
+	    echo "Warning: PID $pid matched but vanished from $PCP_PS_PROG $PCP_PS_ALL_FLAGS output"
+	fi
+    done
+}
+
 # Pass 1 - look in the pmlogger status files for those with -m "note"
 # that suggests they were started by pmlogger_check and friends
 #
@@ -365,6 +392,54 @@ then
     | while read file
     do
 	pid=`echo "$file" | sed -e "s@$PCP_TMP_DIR/pmlogger/@@"`
+	# sanity checks
+	# 1. does this process exist?
+	# 2. is it really pmlogger?
+	# if "no" to either case, remove this (stale) mapfile
+	# and move on ...
+	#
+	if $PCP_PS_PROG -p "$pid" >$tmp/tmp 2>&1
+	then
+	    # ps(1) -p output should be something like this ...
+	    #     PID TTY          TIME CMD
+	    #   14298 ?        00:00:00 pmlogger
+	    # or this (for FreeBSD)
+	    #   PID TT  STAT    TIME COMMAND
+	    # 22839  1  S    0:00.04 /usr/libexec/pcp/bin/pmlogger -N -P ...
+	    # or this (for OpenBSD)
+	    #   PID TT  STAT        TIME COMMAND
+	    #   10619 p1- S        0:00.01 pmlogger -L -c ...
+	    #
+	    if sed -n -e 2p <$tmp/tmp | grep -E -q '( pmlogger$)|(/bin/pmlogger )|( pmlogger )'
+	    then
+		: OK
+	    else
+		if $VERBOSE
+		then
+		    cat $tmp/tmp
+		    echo "Warning: PID $pid is not a pmlogger process, removing $file"
+		fi
+		if $SHOWME
+		then
+		    echo "+ rm $file"
+		else
+		    rm -f "$file"
+		fi
+		continue
+	    fi
+	else
+	    if $VERBOSE
+	    then
+		echo "Warning: PID $pid has vanished, removing $file"
+	    fi
+	    if $SHOWME
+	    then
+		echo "+ rm $file"
+	    else
+		rm -f "$file"
+	    fi
+	    continue
+	fi
 	# timing window here, file may have gone away between
 	# find(1) and awk(1), so just ignore any errors ...
 	#
@@ -377,9 +452,21 @@ NR == 4 && NF == 1	{ if ($1 == "pmlogger_check" ||
 			}'
     done
 fi
+if $VERBOSE
+then
+    if [ -s $tmp/loggers ]
+    then
+	echo "Pass 1: pmloggers from $PCP_TMP_DIR/pmlogger"
+	_debug_report $tmp/loggers
+    else
+	echo "Pass 1: no pmloggers from $PCP_TMP_DIR/pmlogger"
+	ls -l $PCP_TMP_DIR/pmlogger
+    fi
+fi
 
 # Pass 2 - look in the ps(1) output for processes with -m "note" args
 # that suggests they were started by pmlogger_check and friends
+rm -f $tmp/tmp
 $PCP_PS_PROG $PCP_PS_ALL_FLAGS 2>&1 \
 | sed -n -e 's/$/ /' -e '/\/[p]mlogger /{
 /-m *pmlogger_check /bok
@@ -401,12 +488,23 @@ do
     do
 	if grep "^$pid	$dir\$" $tmp/loggers >/dev/null
 	then
-	    : already have this one from the status files
+	    :
 	else
-	    echo "$pid	$dir" >>$tmp/loggers
+	    echo "$pid	$dir" >>$tmp/tmp
 	fi
     done
 done
+if [ -s $tmp/tmp ]
+then
+    if $VERBOSE
+    then
+	echo "Pass 2: add pmloggers from $PCP_PS_PROG $PCP_PS_ALL_FLAGS"
+	_debug_report $tmp/tmp
+    fi
+    cat $tmp/tmp >>$tmp/loggers
+else
+    $VERBOSE && echo "Pass 2: no additional pmloggers from $PCP_PS_PROG $PCP_PS_ALL_FLAGS"
+fi
 
 # Pass 3 - parse the control file(s) culling pmlogger instances that
 # match, as these ones are still under the control of pmlogger_check
@@ -418,36 +516,6 @@ do
     _parse_log_control "$c"
 done
 
-# check for any archives from remote pmloggers via pmproxy or
-# pmlogpush ... if found, synthesize a control file for them
-#
-here=`pwd`
-if cd "$PCP_REMOTE_ARCHIVE_DIR"
-then
-    for _host in *
-    do
-	# TODO - does this need to be smarter?  e.g. check for some
-	# minimal dir contents (.index file?)
-	if [ -d "$_host" ]
-	then
-	    $VERBOSE && echo "Info: processing archives from remote pmlogger on host $_host"
-	    echo '$version=1.1' >$tmp/control
-	    # optional global controls first
-	    [ -f "./control" ] && cat "./control" >>$tmp/control
-	    # optional per-host controls next
-	    [ -f "$_host/control" ] && cat "$_host/control" >>$tmp/control
-	    echo "$_host	n n PCP_REMOTE_ARCHIVE_DIR/$_host +" >>$tmp/control
-	    if $VERY_VERBOSE
-	    then
-		echo >&2 "Synthesized control file ..."
-		cat >&2 $tmp/control
-	    fi
-	    _parse_log_control $tmp/control
-	fi
-    done
-    cd $here
-fi
-
 if [ ! -s $tmp/loggers ]
 then
     # Take early exit if nothing to be done
@@ -475,7 +543,7 @@ do
 	touch $tmp/one-trip
     fi
     echo "Killing (TERM) pmlogger with PID $pid"
-    eval $KILL -s TERM $pid
+    $KILL -s TERM $pid
 done
 
 # Pass 5 - compress archives in orphans' directories
@@ -531,7 +599,8 @@ fi
 #
 sleep 3
 
-# check all the SIGTERM'd loggers really died - if not, use a bigger hammer.
+# Pass 6 - check all the SIGTERM'd loggers really died
+# - if not, use a bigger hammer.
 # 
 if $SHOWME
 then
@@ -542,8 +611,8 @@ else
     do
 	if $PCP_PS_PROG -p "$pid" >/dev/null 2>&1
 	then
-	    echo "Killing (KILL) pmlogger with PID $pid"
-	    eval $KILL -s KILL $pid >/dev/null 2>&1
+	    echo "Pass 6: Killing (KILL) pmlogger with PID $pid"
+	    $KILL -s KILL $pid >/dev/null 2>&1
 	    delay=30        # tenths of a second
 	    while $PCP_PS_PROG -f -p "$pid" >$tmp/alive 2>&1
 	    do
@@ -553,7 +622,7 @@ else
 		    delay=`expr $delay - 1`
 		    continue
 		fi
-		echo "$prog: Error: pmlogger process(es) will not die"
+		echo "$prog: Error: pmlogger process PID $pid will not die"
 		cat $tmp/alive
 		status=1
 		break
