@@ -30,6 +30,8 @@ static struct {
 	{ .group = "series" },		/* METRICS_SERIES */
 	{ .group = "webgroup" },	/* METRICS_WEBGROUP */
 	{ .group = "search" },          /* METRICS_SEARCH */
+	{ .group = "loggroup" },	/* METRICS_LOGGROUP */
+	{ .group = "logpaths" },	/* METRICS_LOGPATHS */
 };
 
 void
@@ -44,7 +46,7 @@ proxylog(pmLogLevel level, sds message, void *arg)
 	state = "- EPOCH - ";
     }
     else
-	state = proxy->slots ? "" : "- DISCONNECTED - ";
+	state = proxy->slotsctx ? "" : "- DISCONNECTED - ";
 
     switch (level) {
     case PMLOG_TRACE:
@@ -82,7 +84,8 @@ proxymetrics(struct proxy *proxy, enum proxy_registry prid)
 	return proxy->metrics[prid];
 
     pmsprintf(path, sizeof(path), "%s%cpmproxy%c%s",
-	    pmGetConfig("PCP_TMP_DIR"), sep, sep, server_metrics[prid].group); //creates a new file
+	    pmGetConfig("PCP_TMP_DIR"), sep, sep, server_metrics[prid].group);
+    /* mmv_stats_registry creates a new file with this path */
     if ((file = strdup(path)) == NULL)
 	return NULL;
 
@@ -93,7 +96,9 @@ proxymetrics(struct proxy *proxy, enum proxy_registry prid)
     else
 	free(file);
     proxy->metrics[prid] = registry;
-    return registry; //once you have a registry you can start adding metrics to it 
+
+    /* once we have a registry we can start adding metrics to it */
+    return registry;
 }
 
 void
@@ -162,6 +167,15 @@ server_init(int portcount, const char *localpath)
     return proxy;
 }
 
+static void
+reset_proxy(struct proxy *proxy)
+{
+    reset_pcp_module(proxy);
+    reset_http_module(proxy);
+    reset_keys_module(proxy);
+    reset_secure_module(proxy);
+}
+
 static void
 signal_handler(uv_signal_t *sighandle, int signum)
 {
@@ -169,17 +183,20 @@ signal_handler(uv_signal_t *sighandle, int signum)
     struct proxy	*proxy = (struct proxy *)handle->data;
     uv_loop_t		*loop = proxy->events;
 
-    if (signum == SIGHUP)
-	return;
-    pmNotifyErr(LOG_INFO, "pmproxy caught %s\n",
-		signum == SIGINT ? "SIGINT" : "SIGTERM");
-    uv_signal_stop(&sigterm);
-    uv_signal_stop(&sigint);
-    uv_signal_stop(&sighup);
-    uv_close((uv_handle_t *)&sigterm, NULL);
-    uv_close((uv_handle_t *)&sigint, NULL);
-    uv_close((uv_handle_t *)&sighup, NULL);
-    uv_stop(loop);
+    if (signum == SIGHUP) {
+	pmNotifyErr(LOG_INFO, "pmproxy caught %s", "SIGHUP");
+	reset_proxy(proxy);
+    } else {
+	pmNotifyErr(LOG_INFO, "pmproxy caught %s",
+		    signum == SIGINT ? "SIGINT" : "SIGTERM");
+	uv_signal_stop(&sigterm);
+	uv_signal_stop(&sigint);
+	uv_signal_stop(&sighup);
+	uv_close((uv_handle_t *)&sigterm, NULL);
+	uv_close((uv_handle_t *)&sigint, NULL);
+	uv_close((uv_handle_t *)&sighup, NULL);
+	uv_stop(loop);
+    }
 }
 
 static void
@@ -204,7 +221,7 @@ void
 on_buffer_alloc(uv_handle_t *handle, size_t suggested_size, uv_buf_t *buf)
 {
     if (pmDebugOptions.desperate)
-	fprintf(stderr, "%s: handle %p buffer allocation of %lld bytes\n",
+	fprintf(stderr, "%s: handle " PRINTF_P_PFX "%p buffer allocation of %lld bytes\n",
 			"on_buffer_alloc", handle, (long long)suggested_size);
 
     if ((buf->base = sdsnewlen(NULL, suggested_size)) != NULL)
@@ -219,7 +236,7 @@ on_client_close(uv_handle_t *handle)
     struct client	*client = (struct client *)handle;
 
     if (pmDebugOptions.context || pmDebugOptions.desperate)
-	fprintf(stderr, "%s: client %p connection closed\n",
+	fprintf(stderr, "%s: client " PRINTF_P_PFX "%p connection closed\n",
 			"on_client_close", client);
 
     client_put(client);
@@ -282,7 +299,7 @@ on_client_write(uv_write_t *writer, int status)
     struct stream_write_baton	*request = (struct stream_write_baton *)writer;
 
     if (pmDebugOptions.af)
-	fprintf(stderr, "%s: completed write [sts=%d] to client %p\n",
+	fprintf(stderr, "%s: completed write [sts=%d] to client " PRINTF_P_PFX "%p\n",
 			"on_client_write", status, client);
 
     if (status == 0) {
@@ -335,7 +352,7 @@ on_write_callback(uv_callback_t *handle, void *data)
 
     (void)handle;
     if (pmDebugOptions.af)
-	fprintf(stderr, "%s: client=%p\n", "on_write_callback", client);
+	fprintf(stderr, "%s: client=" PRINTF_P_PFX "%p\n", "on_write_callback", client);
 
     if (client->stream.secure == 0) {
 	sts = uv_write(&request->writer, (uv_stream_t *)&client->stream,
@@ -366,12 +383,12 @@ client_write(struct client *client, sds buffer, sds suffix)
 
     if ((request = calloc(1, sizeof(struct stream_write_baton))) != NULL) {
 	if (pmDebugOptions.af)
-	    fprintf(stderr, "%s: sending %ld bytes [0] to client %p\n",
+	    fprintf(stderr, "%s: sending %ld bytes [0] to client " PRINTF_P_PFX "%p\n",
 			"client_write", (long)sdslen(buffer), client);
 	request->buffer[nbuffers++] = uv_buf_init(buffer, sdslen(buffer));
 	if (suffix != NULL) {
 	    if (pmDebugOptions.af)
-		fprintf(stderr, "%s: sending %ld bytes [1] to client %p\n",
+		fprintf(stderr, "%s: sending %ld bytes [1] to client " PRINTF_P_PFX "%p\n",
 			"client_write", (long)sdslen(suffix), client);
 	    request->buffer[nbuffers++] = uv_buf_init(suffix, sdslen(suffix));
 	}
@@ -379,7 +396,7 @@ client_write(struct client *client, sds buffer, sds suffix)
 	request->writer.data = client;
 	request->callback = on_client_write;
 
-	/* client must not get freed while waiting for the write callback to fire */
+	/* client must not be freed while waiting for the write callback */
 	client_get(client);
 	uv_callback_fire(&proxy->write_callbacks, request, NULL);
     } else {
@@ -444,7 +461,7 @@ on_protocol_read(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
     else {
 	if (pmDebugOptions.af)
 	    fprintf(stderr, "%s: unknown protocol key '%c' (0x%x)"
-			    " - disconnecting client %p\n", "on_protocol_read",
+			    " - disconnecting client " PRINTF_P_PFX "%p\n", "on_protocol_read",
 		    *buf->base, (unsigned int)*buf->base, proxy);
 	client_close(client);
     }
@@ -474,7 +491,7 @@ on_client_read(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
     } else if (nread < 0) {
 	if (pmDebugOptions.af)
 	    fprintf(stderr, "%s: read error %ld "
-		    "- disconnecting client %p\n", "on_client_read",
+		    "- disconnecting client " PRINTF_P_PFX "%p\n", "on_client_read",
 		    (long)nread, client);
 	client_close(client);
     }
@@ -502,7 +519,7 @@ on_client_connection(uv_stream_t *stream, int status)
 	return;
     }
     if (pmDebugOptions.context || pmDebugOptions.af)
-	fprintf(stderr, "%s: accept new client %p\n",
+	fprintf(stderr, "%s: accept new client " PRINTF_P_PFX "%p\n",
 			"on_client_connection", client);
 
     /* prepare per-client lock for reference counting */
@@ -775,8 +792,8 @@ static void
 close_proxy(struct proxy *proxy)
 {
     close_pcp_module(proxy);
-    close_http_module(proxy);
     close_keys_module(proxy);
+    close_http_module(proxy);
     close_secure_module(proxy);
 }
 
@@ -803,6 +820,7 @@ shutdown_ports(void *arg)
 	}
     }
     proxy->nservers = 0;
+    proxy->map = NULL;
 
     close_proxy(proxy);
     if (proxy->config) {
