@@ -644,7 +644,7 @@ class pmConfig(object):
         except pmapi.pmErr as error:
             if hasattr(self.util, 'ignore_incompat') and self.util.ignore_incompat:
                 return
-            sys.stderr.write("Invalid metric %s (%s).\n" % (metric, str(error)))
+            sys.stderr.write("Incompatible metric %s (%s).\n" % (metric, str(error)))
             sys.exit(1)
 
     def ignore_unknown_metrics(self):
@@ -764,8 +764,9 @@ class pmConfig(object):
                         err = "Unidentified error"
                     finally:
                         if err:
-                            sys.stderr.write("Failed to register derived metric:\n%s.\n" % err)
-                            sys.exit(1)
+                            sys.stderr.write("Failed to register derived metric:\n%s\n" % err.strip())
+                            if not self.ignore_unknown_metrics():
+                                sys.exit(1)
 
         if not hasattr(self.util, 'leaf_only') or not self.util.leaf_only:
             # Prepare for non-leaf metrics while preserving metric order
@@ -803,6 +804,7 @@ class pmConfig(object):
         metrics = self.util.metrics
         self.util.metrics = OrderedDict()
 
+        some_unknown = False
         for metric in metrics:
             try:
                 l = len(self.pmids)
@@ -814,8 +816,12 @@ class pmConfig(object):
                 else:
                     self.util.metrics[metric] = metrics[metric]
             except pmapi.pmErr as error:
-                sys.stderr.write("Invalid metric %s (%s).\n" % (metric, str(error)))
-                sys.exit(1)
+                sys.stderr.write("Unknown metric %s (%s).\n" % (metric, str(error)))
+                some_unknown = True
+
+        # Exit if some were unknown, unless --ignore-unknown
+        if some_unknown and not self.ignore_unknown_metrics():
+            sys.exit(1)
 
         # Exit if no metrics with specified instances found
         if not self.insts and not self.ignore_unknown_metrics():
@@ -826,6 +832,11 @@ class pmConfig(object):
                 print(self.util.instances)
             sys.exit(1)
 
+        # Exit if no metric was valid
+        if not self.util.metrics:
+            sys.stderr.write("All metrics are unknown, quitting.\n")
+            sys.exit(1)
+
         # Dynamically set fetchgroup max instances
         # if not specified explicitly by the caller
         dynamic_insts = not max_insts
@@ -1147,9 +1158,9 @@ class pmConfig(object):
 
         # Watch for end time in uninterpolated mode
         if not self.util.interpol:
-            sample = self.util.pmfg_ts().strftime('%s')
-            finish = self.util.opts.pmGetOptionFinish()
-            if float(sample) > float(finish):
+            sample_ts = self.util.pmfg_ts().timestamp()
+            finish_ts = float(self.util.opts.pmGetOptionFinish())
+            if sample_ts > finish_ts:
                 return -2
 
         # Handle any PMCD state change notification
@@ -1168,7 +1179,7 @@ class pmConfig(object):
         self._round += 1
 
         if not self._init_ts:
-            self._init_ts = float(self.util.pmfg_ts().strftime("%s.%f"))
+            self._init_ts = self.util.pmfg_ts().timestamp()
 
         wakeup = self._init_ts + float(self.util.interval) * self._round
 
