@@ -46,14 +46,14 @@ setup(name = 'pcp',
         Extension('cmmv', ['mmv.c'], libraries = ['pcp_mmv', 'pcp']),
     ],
     keywords = ['performance', 'analysis', 'monitoring' ],
-    platforms = [ 'Windows', 'Linux', 'FreeBSD', 'NetBSD', 'OpenBSD', 'Solaris', 'Mac OS X', 'AIX' ],
+    platforms = [ 'Windows', 'Linux', 'FreeBSD', 'NetBSD', 'OpenBSD', 'Solaris', 'macOS', 'AIX' ],
     classifiers = [
         'Development Status :: 5 - Production/Stable',
         'Intended Audience :: Developers',
         'Intended Audience :: System Administrators',
         'Intended Audience :: Information Technology',
         'Natural Language :: English',
-        'Operating System :: MacOS :: MacOS X',
+        'Operating System :: macOS :: macOS',
         'Operating System :: Microsoft :: Windows',
         'Operating System :: POSIX',
         'Operating System :: POSIX :: AIX',
